# coding: utf-8

"""
    CustomerManagementService

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import datetime
from pydantic import BaseModel, ConfigDict, Field, StrictBool, StrictFloat, StrictInt, StrictStr
from typing import Any, ClassVar, Dict, List, Optional, Union, Set
from openapi_client.models.customer.account_financial_status import AccountFinancialStatus
from openapi_client.models.customer.account_life_cycle_status import AccountLifeCycleStatus
from openapi_client.models.customer.account_tax_certificate import AccountTaxCertificate
from openapi_client.models.customer.address import Address
from openapi_client.models.customer.auto_tag_type import AutoTagType
from openapi_client.models.customer.currency_code import CurrencyCode
from openapi_client.models.customer.customer_info import CustomerInfo
from openapi_client.models.customer.key_value_pair_ofstring_andstring import KeyValuePairOfstringAndstring
from openapi_client.models.customer.language_type import LanguageType
from openapi_client.models.customer.payment_method_type import PaymentMethodType
from openapi_client.models.customer.time_zone_type import TimeZoneType
from typing_extensions import Self

class AdvertiserAccount(BaseModel):
    """
    AdvertiserAccount
    """ # noqa: E501
    bill_to_customer_id: Optional[StrictStr] = Field(default=None, alias="BillToCustomerId")
    currency_code: Optional[CurrencyCode] = Field(default=None, alias="CurrencyCode")
    account_financial_status: Optional[AccountFinancialStatus] = Field(default=None, alias="AccountFinancialStatus")
    id: Optional[StrictStr] = Field(default=None, alias="Id")
    language: Optional[LanguageType] = Field(default=None, alias="Language")
    last_modified_by_user_id: Optional[StrictStr] = Field(default=None, alias="LastModifiedByUserId")
    last_modified_time: Optional[datetime] = Field(default=None, alias="LastModifiedTime")
    name: Optional[StrictStr] = Field(default=None, alias="Name")
    number: Optional[StrictStr] = Field(default=None, alias="Number")
    parent_customer_id: Optional[StrictStr] = Field(default=None, alias="ParentCustomerId")
    payment_method_id: Optional[StrictStr] = Field(default=None, alias="PaymentMethodId")
    payment_method_type: Optional[PaymentMethodType] = Field(default=None, alias="PaymentMethodType")
    primary_user_id: Optional[StrictStr] = Field(default=None, alias="PrimaryUserId")
    account_life_cycle_status: Optional[AccountLifeCycleStatus] = Field(default=None, alias="AccountLifeCycleStatus")
    time_stamp: Optional[StrictStr] = Field(default=None, alias="TimeStamp")
    time_zone: Optional[TimeZoneType] = Field(default=None, alias="TimeZone")
    pause_reason: Optional[StrictInt] = Field(default=None, alias="PauseReason")
    forward_compatibility_map: Optional[List[Optional[KeyValuePairOfstringAndstring]]] = Field(default=None, alias="ForwardCompatibilityMap")
    linked_agencies: Optional[List[Optional[CustomerInfo]]] = Field(default=None, alias="LinkedAgencies")
    sales_house_customer_id: Optional[StrictStr] = Field(default=None, alias="SalesHouseCustomerId")
    tax_information: Optional[List[Optional[KeyValuePairOfstringAndstring]]] = Field(default=None, alias="TaxInformation")
    back_up_payment_instrument_id: Optional[StrictStr] = Field(default=None, alias="BackUpPaymentInstrumentId")
    billing_threshold_amount: Optional[StrictFloat] = Field(default=None, alias="BillingThresholdAmount")
    business_address: Optional[Address] = Field(default=None, alias="BusinessAddress")
    auto_tag_type: Optional[AutoTagType] = Field(default=None, alias="AutoTagType")
    sold_to_payment_instrument_id: Optional[StrictStr] = Field(default=None, alias="SoldToPaymentInstrumentId")
    tax_certificate: Optional[AccountTaxCertificate] = Field(default=None, alias="TaxCertificate")
    account_mode: Optional[StrictStr] = Field(default=None, alias="AccountMode")
    __properties: ClassVar[List[str]] = ["BillToCustomerId", "CurrencyCode", "AccountFinancialStatus", "Id", "Language", "LastModifiedByUserId", "LastModifiedTime", "Name", "Number", "ParentCustomerId", "PaymentMethodId", "PaymentMethodType", "PrimaryUserId", "AccountLifeCycleStatus", "TimeStamp", "TimeZone", "PauseReason", "ForwardCompatibilityMap", "LinkedAgencies", "SalesHouseCustomerId", "TaxInformation", "BackUpPaymentInstrumentId", "BillingThresholdAmount", "BusinessAddress", "AutoTagType", "SoldToPaymentInstrumentId", "TaxCertificate", "AccountMode"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )
	

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of each item in forward_compatibility_map (list)
        _items = []
        if self.forward_compatibility_map:
            for _item_forward_compatibility_map in self.forward_compatibility_map:
                if _item_forward_compatibility_map:
                    _items.append(_item_forward_compatibility_map.to_dict())
            _dict['ForwardCompatibilityMap'] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in linked_agencies (list)
        _items = []
        if self.linked_agencies:
            for _item_linked_agencies in self.linked_agencies:
                if _item_linked_agencies:
                    _items.append(_item_linked_agencies.to_dict())
            _dict['LinkedAgencies'] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in tax_information (list)
        _items = []
        if self.tax_information:
            for _item_tax_information in self.tax_information:
                if _item_tax_information:
                    _items.append(_item_tax_information.to_dict())
            _dict['TaxInformation'] = _items
        # override the default output from pydantic by calling `to_dict()` of business_address
        if self.business_address:
            _dict['BusinessAddress'] = self.business_address.to_dict()
        # override the default output from pydantic by calling `to_dict()` of tax_certificate
        if self.tax_certificate:
            _dict['TaxCertificate'] = self.tax_certificate.to_dict()
        # set to None if bill_to_customer_id (nullable) is None
        # and model_fields_set contains the field
        if self.bill_to_customer_id is None and "bill_to_customer_id" in self.model_fields_set:
            _dict['BillToCustomerId'] = None

        # set to None if currency_code (nullable) is None
        # and model_fields_set contains the field
        if self.currency_code is None and "currency_code" in self.model_fields_set:
            _dict['CurrencyCode'] = None

        # set to None if account_financial_status (nullable) is None
        # and model_fields_set contains the field
        if self.account_financial_status is None and "account_financial_status" in self.model_fields_set:
            _dict['AccountFinancialStatus'] = None

        # set to None if id (nullable) is None
        # and model_fields_set contains the field
        if self.id is None and "id" in self.model_fields_set:
            _dict['Id'] = None

        # set to None if language (nullable) is None
        # and model_fields_set contains the field
        if self.language is None and "language" in self.model_fields_set:
            _dict['Language'] = None

        # set to None if last_modified_by_user_id (nullable) is None
        # and model_fields_set contains the field
        if self.last_modified_by_user_id is None and "last_modified_by_user_id" in self.model_fields_set:
            _dict['LastModifiedByUserId'] = None

        # set to None if last_modified_time (nullable) is None
        # and model_fields_set contains the field
        if self.last_modified_time is None and "last_modified_time" in self.model_fields_set:
            _dict['LastModifiedTime'] = None

        # set to None if name (nullable) is None
        # and model_fields_set contains the field
        if self.name is None and "name" in self.model_fields_set:
            _dict['Name'] = None

        # set to None if number (nullable) is None
        # and model_fields_set contains the field
        if self.number is None and "number" in self.model_fields_set:
            _dict['Number'] = None

        # set to None if parent_customer_id (nullable) is None
        # and model_fields_set contains the field
        if self.parent_customer_id is None and "parent_customer_id" in self.model_fields_set:
            _dict['ParentCustomerId'] = None

        # set to None if payment_method_id (nullable) is None
        # and model_fields_set contains the field
        if self.payment_method_id is None and "payment_method_id" in self.model_fields_set:
            _dict['PaymentMethodId'] = None

        # set to None if payment_method_type (nullable) is None
        # and model_fields_set contains the field
        if self.payment_method_type is None and "payment_method_type" in self.model_fields_set:
            _dict['PaymentMethodType'] = None

        # set to None if primary_user_id (nullable) is None
        # and model_fields_set contains the field
        if self.primary_user_id is None and "primary_user_id" in self.model_fields_set:
            _dict['PrimaryUserId'] = None

        # set to None if account_life_cycle_status (nullable) is None
        # and model_fields_set contains the field
        if self.account_life_cycle_status is None and "account_life_cycle_status" in self.model_fields_set:
            _dict['AccountLifeCycleStatus'] = None

        # set to None if time_stamp (nullable) is None
        # and model_fields_set contains the field
        if self.time_stamp is None and "time_stamp" in self.model_fields_set:
            _dict['TimeStamp'] = None

        # set to None if time_zone (nullable) is None
        # and model_fields_set contains the field
        if self.time_zone is None and "time_zone" in self.model_fields_set:
            _dict['TimeZone'] = None

        # set to None if pause_reason (nullable) is None
        # and model_fields_set contains the field
        if self.pause_reason is None and "pause_reason" in self.model_fields_set:
            _dict['PauseReason'] = None

        # set to None if forward_compatibility_map (nullable) is None
        # and model_fields_set contains the field
        if self.forward_compatibility_map is None and "forward_compatibility_map" in self.model_fields_set:
            _dict['ForwardCompatibilityMap'] = None

        # set to None if linked_agencies (nullable) is None
        # and model_fields_set contains the field
        if self.linked_agencies is None and "linked_agencies" in self.model_fields_set:
            _dict['LinkedAgencies'] = None

        # set to None if sales_house_customer_id (nullable) is None
        # and model_fields_set contains the field
        if self.sales_house_customer_id is None and "sales_house_customer_id" in self.model_fields_set:
            _dict['SalesHouseCustomerId'] = None

        # set to None if tax_information (nullable) is None
        # and model_fields_set contains the field
        if self.tax_information is None and "tax_information" in self.model_fields_set:
            _dict['TaxInformation'] = None

        # set to None if back_up_payment_instrument_id (nullable) is None
        # and model_fields_set contains the field
        if self.back_up_payment_instrument_id is None and "back_up_payment_instrument_id" in self.model_fields_set:
            _dict['BackUpPaymentInstrumentId'] = None

        # set to None if billing_threshold_amount (nullable) is None
        # and model_fields_set contains the field
        if self.billing_threshold_amount is None and "billing_threshold_amount" in self.model_fields_set:
            _dict['BillingThresholdAmount'] = None

        # set to None if business_address (nullable) is None
        # and model_fields_set contains the field
        if self.business_address is None and "business_address" in self.model_fields_set:
            _dict['BusinessAddress'] = None

        # set to None if auto_tag_type (nullable) is None
        # and model_fields_set contains the field
        if self.auto_tag_type is None and "auto_tag_type" in self.model_fields_set:
            _dict['AutoTagType'] = None

        # set to None if sold_to_payment_instrument_id (nullable) is None
        # and model_fields_set contains the field
        if self.sold_to_payment_instrument_id is None and "sold_to_payment_instrument_id" in self.model_fields_set:
            _dict['SoldToPaymentInstrumentId'] = None

        # set to None if tax_certificate (nullable) is None
        # and model_fields_set contains the field
        if self.tax_certificate is None and "tax_certificate" in self.model_fields_set:
            _dict['TaxCertificate'] = None

        # set to None if account_mode (nullable) is None
        # and model_fields_set contains the field
        if self.account_mode is None and "account_mode" in self.model_fields_set:
            _dict['AccountMode'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of AdvertiserAccount from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "BillToCustomerId": obj.get("BillToCustomerId") if obj.get("BillToCustomerId") is not None else None,
                        "CurrencyCode": obj.get("CurrencyCode") if obj.get("CurrencyCode") is not None else None,
                        "AccountFinancialStatus": obj.get("AccountFinancialStatus") if obj.get("AccountFinancialStatus") is not None else None,
                        "Id": obj.get("Id") if obj.get("Id") is not None else None,
                        "Language": obj.get("Language") if obj.get("Language") is not None else None,
                        "LastModifiedByUserId": obj.get("LastModifiedByUserId") if obj.get("LastModifiedByUserId") is not None else None,
                        "LastModifiedTime": obj.get("LastModifiedTime") if obj.get("LastModifiedTime") is not None else None,
                        "Name": obj.get("Name") if obj.get("Name") is not None else None,
                        "Number": obj.get("Number") if obj.get("Number") is not None else None,
                        "ParentCustomerId": obj.get("ParentCustomerId") if obj.get("ParentCustomerId") is not None else None,
                        "PaymentMethodId": obj.get("PaymentMethodId") if obj.get("PaymentMethodId") is not None else None,
                        "PaymentMethodType": obj.get("PaymentMethodType") if obj.get("PaymentMethodType") is not None else None,
                        "PrimaryUserId": obj.get("PrimaryUserId") if obj.get("PrimaryUserId") is not None else None,
                        "AccountLifeCycleStatus": obj.get("AccountLifeCycleStatus") if obj.get("AccountLifeCycleStatus") is not None else None,
                        "TimeStamp": obj.get("TimeStamp") if obj.get("TimeStamp") is not None else None,
                        "TimeZone": obj.get("TimeZone") if obj.get("TimeZone") is not None else None,
                        "PauseReason": obj.get("PauseReason") if obj.get("PauseReason") is not None else None,
                        "ForwardCompatibilityMap": [KeyValuePairOfstringAndstring.from_dict(_item) for _item in obj["ForwardCompatibilityMap"]] if obj.get("ForwardCompatibilityMap") is not None else None,
                        "LinkedAgencies": [CustomerInfo.from_dict(_item) for _item in obj["LinkedAgencies"]] if obj.get("LinkedAgencies") is not None else None,
                        "SalesHouseCustomerId": obj.get("SalesHouseCustomerId") if obj.get("SalesHouseCustomerId") is not None else None,
                        "TaxInformation": [KeyValuePairOfstringAndstring.from_dict(_item) for _item in obj["TaxInformation"]] if obj.get("TaxInformation") is not None else None,
                        "BackUpPaymentInstrumentId": obj.get("BackUpPaymentInstrumentId") if obj.get("BackUpPaymentInstrumentId") is not None else None,
                        "BillingThresholdAmount": obj.get("BillingThresholdAmount") if obj.get("BillingThresholdAmount") is not None else None,
                        "BusinessAddress": Address.from_dict(obj["BusinessAddress"]) if obj.get("BusinessAddress") is not None else None,
                        "AutoTagType": obj.get("AutoTagType") if obj.get("AutoTagType") is not None else None,
                        "SoldToPaymentInstrumentId": obj.get("SoldToPaymentInstrumentId") if obj.get("SoldToPaymentInstrumentId") is not None else None,
                        "TaxCertificate": AccountTaxCertificate.from_dict(obj["TaxCertificate"]) if obj.get("TaxCertificate") is not None else None,
                        "AccountMode": obj.get("AccountMode") if obj.get("AccountMode") is not None else None
        })
        return _obj



# coding: utf-8

"""
    BulkService

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import datetime
from pydantic import BaseModel, ConfigDict, Field, StrictBool, StrictFloat, StrictInt, StrictStr
from typing import Any, ClassVar, Dict, List, Optional, Union, Set
from openapi_client.models.bulk.campaign_scope import CampaignScope
from openapi_client.models.bulk.compression_type import CompressionType
from openapi_client.models.bulk.data_scope import DataScope
from openapi_client.models.bulk.download_entity import DownloadEntity
from openapi_client.models.bulk.download_file_type import DownloadFileType
from typing_extensions import Self

class DownloadCampaignsByCampaignIdsRequest(BaseModel):
    """
    DownloadCampaignsByCampaignIdsRequest
    """ # noqa: E501
    campaigns: Optional[List[Optional[CampaignScope]]] = Field(default=None, alias="Campaigns")
    last_sync_time_in_utc: Optional[datetime] = Field(default=None, alias="LastSyncTimeInUTC")
    download_entities: Optional[List[Optional[DownloadEntity]]] = Field(default=None, alias="DownloadEntities")
    data_scope: Optional[DataScope] = Field(default=None, alias="DataScope")
    download_file_type: Optional[DownloadFileType] = Field(default=None, alias="DownloadFileType")
    format_version: Optional[StrictStr] = Field(default=None, alias="FormatVersion")
    compression_type: Optional[CompressionType] = Field(default=None, alias="CompressionType")
    __properties: ClassVar[List[str]] = ["Campaigns", "LastSyncTimeInUTC", "DownloadEntities", "DataScope", "DownloadFileType", "FormatVersion", "CompressionType"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )
	

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of each item in campaigns (list)
        _items = []
        if self.campaigns:
            for _item_campaigns in self.campaigns:
                if _item_campaigns:
                    _items.append(_item_campaigns.to_dict())
            _dict['Campaigns'] = _items
        # set to None if campaigns (nullable) is None
        # and model_fields_set contains the field
        if self.campaigns is None and "campaigns" in self.model_fields_set:
            _dict['Campaigns'] = None

        # set to None if last_sync_time_in_utc (nullable) is None
        # and model_fields_set contains the field
        if self.last_sync_time_in_utc is None and "last_sync_time_in_utc" in self.model_fields_set:
            _dict['LastSyncTimeInUTC'] = None

        # set to None if download_entities (nullable) is None
        # and model_fields_set contains the field
        if self.download_entities is None and "download_entities" in self.model_fields_set:
            _dict['DownloadEntities'] = None

        # set to None if data_scope (nullable) is None
        # and model_fields_set contains the field
        if self.data_scope is None and "data_scope" in self.model_fields_set:
            _dict['DataScope'] = None

        # set to None if download_file_type (nullable) is None
        # and model_fields_set contains the field
        if self.download_file_type is None and "download_file_type" in self.model_fields_set:
            _dict['DownloadFileType'] = None

        # set to None if format_version (nullable) is None
        # and model_fields_set contains the field
        if self.format_version is None and "format_version" in self.model_fields_set:
            _dict['FormatVersion'] = None

        # set to None if compression_type (nullable) is None
        # and model_fields_set contains the field
        if self.compression_type is None and "compression_type" in self.model_fields_set:
            _dict['CompressionType'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of DownloadCampaignsByCampaignIdsRequest from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "Campaigns": [CampaignScope.from_dict(_item) for _item in obj["Campaigns"]] if obj.get("Campaigns") is not None else None,
                        "LastSyncTimeInUTC": obj.get("LastSyncTimeInUTC") if obj.get("LastSyncTimeInUTC") is not None else None,
                        "DownloadEntities": obj.get("DownloadEntities"),
                        "DataScope": obj.get("DataScope") if obj.get("DataScope") is not None else None,
                        "DownloadFileType": obj.get("DownloadFileType") if obj.get("DownloadFileType") is not None else None,
                        "FormatVersion": obj.get("FormatVersion") if obj.get("FormatVersion") is not None else None,
                        "CompressionType": obj.get("CompressionType") if obj.get("CompressionType") is not None else None
        })
        return _obj



# coding: utf-8

"""
    CampaignManagementService

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import datetime
from pydantic import BaseModel, ConfigDict, Field, StrictBool, StrictFloat, StrictInt, StrictStr
from typing import Any, ClassVar, Dict, List, Optional, Union, Set
from openapi_client.models.campaign.price_qualifier import PriceQualifier
from openapi_client.models.campaign.price_unit import PriceUnit
from typing_extensions import Self

class PriceTableRow(BaseModel):
    """
    PriceTableRow
    """ # noqa: E501
    header: Optional[StrictStr] = Field(default=None, alias="Header")
    description: Optional[StrictStr] = Field(default=None, alias="Description")
    final_urls: Optional[List[StrictStr]] = Field(default=None, alias="FinalUrls")
    final_mobile_urls: Optional[List[StrictStr]] = Field(default=None, alias="FinalMobileUrls")
    price: Optional[StrictFloat] = Field(default=None, alias="Price")
    currency_code: Optional[StrictStr] = Field(default=None, alias="CurrencyCode")
    price_unit: Optional[PriceUnit] = Field(default=None, alias="PriceUnit")
    price_qualifier: Optional[PriceQualifier] = Field(default=None, alias="PriceQualifier")
    terms_and_conditions: Optional[StrictStr] = Field(default=None, alias="TermsAndConditions")
    terms_and_conditions_url: Optional[StrictStr] = Field(default=None, alias="TermsAndConditionsUrl")
    __properties: ClassVar[List[str]] = ["Header", "Description", "FinalUrls", "FinalMobileUrls", "Price", "CurrencyCode", "PriceUnit", "PriceQualifier", "TermsAndConditions", "TermsAndConditionsUrl"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )
	

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # set to None if header (nullable) is None
        # and model_fields_set contains the field
        if self.header is None and "header" in self.model_fields_set:
            _dict['Header'] = None

        # set to None if description (nullable) is None
        # and model_fields_set contains the field
        if self.description is None and "description" in self.model_fields_set:
            _dict['Description'] = None

        # set to None if final_urls (nullable) is None
        # and model_fields_set contains the field
        if self.final_urls is None and "final_urls" in self.model_fields_set:
            _dict['FinalUrls'] = None

        # set to None if final_mobile_urls (nullable) is None
        # and model_fields_set contains the field
        if self.final_mobile_urls is None and "final_mobile_urls" in self.model_fields_set:
            _dict['FinalMobileUrls'] = None

        # set to None if price (nullable) is None
        # and model_fields_set contains the field
        if self.price is None and "price" in self.model_fields_set:
            _dict['Price'] = None

        # set to None if currency_code (nullable) is None
        # and model_fields_set contains the field
        if self.currency_code is None and "currency_code" in self.model_fields_set:
            _dict['CurrencyCode'] = None

        # set to None if price_unit (nullable) is None
        # and model_fields_set contains the field
        if self.price_unit is None and "price_unit" in self.model_fields_set:
            _dict['PriceUnit'] = None

        # set to None if price_qualifier (nullable) is None
        # and model_fields_set contains the field
        if self.price_qualifier is None and "price_qualifier" in self.model_fields_set:
            _dict['PriceQualifier'] = None

        # set to None if terms_and_conditions (nullable) is None
        # and model_fields_set contains the field
        if self.terms_and_conditions is None and "terms_and_conditions" in self.model_fields_set:
            _dict['TermsAndConditions'] = None

        # set to None if terms_and_conditions_url (nullable) is None
        # and model_fields_set contains the field
        if self.terms_and_conditions_url is None and "terms_and_conditions_url" in self.model_fields_set:
            _dict['TermsAndConditionsUrl'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of PriceTableRow from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "Header": obj.get("Header") if obj.get("Header") is not None else None,
                        "Description": obj.get("Description") if obj.get("Description") is not None else None,
                        "FinalUrls": obj.get("FinalUrls"),
                        "FinalMobileUrls": obj.get("FinalMobileUrls"),
                        "Price": obj.get("Price") if obj.get("Price") is not None else None,
                        "CurrencyCode": obj.get("CurrencyCode") if obj.get("CurrencyCode") is not None else None,
                        "PriceUnit": obj.get("PriceUnit") if obj.get("PriceUnit") is not None else None,
                        "PriceQualifier": obj.get("PriceQualifier") if obj.get("PriceQualifier") is not None else None,
                        "TermsAndConditions": obj.get("TermsAndConditions") if obj.get("TermsAndConditions") is not None else None,
                        "TermsAndConditionsUrl": obj.get("TermsAndConditionsUrl") if obj.get("TermsAndConditionsUrl") is not None else None
        })
        return _obj



from abc import ABCMeta, abstractmethod


class _BulkMapping(metaclass=ABCMeta):
    @abstractmethod
    def convert_to_csv(self, entity, row_values):
        raise NotImplementedError()

    @abstractmethod
    def convert_to_entity(self, row_values, entity):
        raise NotImplementedError()


class _SingleFieldBulkMapping(_BulkMapping, metaclass=ABCMeta):
    def __init__(self, csv_to_field, filed_to_csv):
        self._csv_to_field = csv_to_field
        self._field_to_csv = filed_to_csv

    @property
    def csv_to_field(self):
        """

        :rtype: str -> T
        """

        return self._csv_to_field

    @property
    def field_to_csv(self):
        """

        :rtype: T -> str
        :return:
        """
        return self._field_to_csv

    def convert_to_csv(self, entity, row_values):
        if self.field_to_csv is None:
            return None
        row_values[self.parse_header(entity)] = self.field_to_csv(entity)

    def convert_to_entity(self, row_values, entity):
        # Bulk file can have fewer column than SDK knows, so to have the ability to read old file, if cannot find column
        # just return None for that column, if it is mandatory, then will throw exception later, if not, then pass.

        self.csv_to_field(entity, row_values.try_get_value(self.parse_header(entity))[1])

    @abstractmethod
    def parse_header(self, entity):
        raise NotImplementedError()


class _SimpleBulkMapping(_SingleFieldBulkMapping):
    def __init__(self, header, csv_to_field, field_to_csv=None):
        super(_SimpleBulkMapping, self).__init__(csv_to_field, field_to_csv)
        self._header = header

    @property
    def header(self):
        """

        :rtype: str

        """
        return self._header

    def parse_header(self, entity):
        return self.header


class _DynamicColumnNameMapping(_SingleFieldBulkMapping):
    def __init__(self, header_func, csv_to_field, field_to_csv=None):
        super(_DynamicColumnNameMapping, self).__init__(csv_to_field, field_to_csv)
        self._header_func = header_func

    @property
    def header_func(self):
        """

        :rtype: T -> str
        """
        return self._header_func

    def parse_header(self, entity):
        return self.header_func(entity)


class _ComplexBulkMapping(_BulkMapping):
    def __init__(self, entity_to_csv, csv_to_entity):
        self._entity_to_csv = entity_to_csv
        self._csv_to_entity = csv_to_entity

    @property
    def csv_to_entity(self):
        """

        :rtype: _RowValues -> T
        """

        return self._csv_to_entity

    @property
    def entity_to_csv(self):
        """

        :rtype: T -> _RowValues
        """

        return self._entity_to_csv

    def convert_to_csv(self, entity, row_values):
        self.entity_to_csv(entity, row_values)

    def convert_to_entity(self, row_values, entity):
        self.csv_to_entity(row_values, entity)



# coding: utf-8

"""
    CampaignManagementService

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import json
from enum import Enum
from typing_extensions import Self

class AccountPropertyName(str, Enum):
    NONE = 'None'

    TRACKINGURLTEMPLATE = 'TrackingUrlTemplate'

    MSCLKIDAUTOTAGGINGENABLED = 'MSCLKIDAutoTaggingEnabled'

    ADCLICKPARALLELTRACKING = 'AdClickParallelTracking'

    FINALURLSUFFIX = 'FinalUrlSuffix'

    INCLUDEVIEWTHROUGHCONVERSIONS = 'IncludeViewThroughConversions'

    PROFILEEXPANSIONENABLED = 'ProfileExpansionEnabled'

    ALLOWIMAGEAUTORETRIEVE = 'AllowImageAutoRetrieve'

    AUTOAPPLYRECOMMENDATIONS = 'AutoApplyRecommendations'

    INCLUDEAUTOBIDDINGVIEWTHROUGHCONVERSIONS = 'IncludeAutoBiddingViewThroughConversions'

    AUTOBIDDINGVIEWTHROUGHCONVERSIONSVALUEATTRIBUTIONWEIGHT = 'AutoBiddingViewThroughConversionsValueAttributionWeight'

    LOOPBACKWINDOWFORVIEWTHROUGHCONVERSIONS = 'LoopBackWindowForViewThroughConversions'

    BUSINESSATTRIBUTES = 'BusinessAttributes'

    ENABLEMMAUNDERDSAADGROUPS = 'EnableMMAUnderDSAAdgroups'

    OPTOUTFROMMCM = 'OptOutFromMCM'

    NETFLIXTCACCEPTED = 'NetflixTCAccepted'

    BLOCKEDCONTENTSEGMENTS = 'BlockedContentSegments'

    ASSETAIENHANCEMENTOPTOUT = 'AssetAIEnhancementOptout'

	
    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of AccountPropertyName from a JSON string"""
        return cls(json.loads(json_str))





# coding: utf-8

"""
    AdInsightService

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import datetime
from pydantic import BaseModel, ConfigDict, Field, StrictBool, StrictFloat, StrictInt, StrictStr
from typing import Any, ClassVar, Dict, List, Optional, Union, Set
from openapi_client.models.adinsight.currency import Currency
from openapi_client.models.adinsight.feed import Feed
from openapi_client.models.adinsight.radius_target import RadiusTarget
from openapi_client.models.adinsight.selection_of_age_enum import SelectionOfAgeEnum
from openapi_client.models.adinsight.selection_of_gender_enum import SelectionOfGenderEnum
from openapi_client.models.adinsight.selection_oflong import SelectionOflong
from typing_extensions import Self

class GetAudienceFullEstimationRequest(BaseModel):
    """
    GetAudienceFullEstimationRequest
    """ # noqa: E501
    age: Optional[SelectionOfAgeEnum] = Field(default=None, alias="Age")
    gender: Optional[SelectionOfGenderEnum] = Field(default=None, alias="Gender")
    audience: Optional[SelectionOflong] = Field(default=None, alias="Audience")
    company: Optional[SelectionOflong] = Field(default=None, alias="Company")
    industry: Optional[SelectionOflong] = Field(default=None, alias="Industry")
    job_function: Optional[SelectionOflong] = Field(default=None, alias="JobFunction")
    location: Optional[SelectionOflong] = Field(default=None, alias="Location")
    radius_targets: Optional[List[Optional[RadiusTarget]]] = Field(default=None, alias="RadiusTargets")
    campaign_bidding_strategy: Optional[StrictInt] = Field(default=None, alias="CampaignBiddingStrategy")
    campaign_sub_type: Optional[StrictInt] = Field(default=None, alias="CampaignSubType")
    feed: Optional[Feed] = Field(default=None, alias="Feed")
    daily_budget: Optional[StrictFloat] = Field(default=None, alias="DailyBudget")
    bid: Optional[StrictFloat] = Field(default=None, alias="Bid")
    currency: Optional[Currency] = Field(default=None, alias="Currency")
    __properties: ClassVar[List[str]] = ["Age", "Gender", "Audience", "Company", "Industry", "JobFunction", "Location", "RadiusTargets", "CampaignBiddingStrategy", "CampaignSubType", "Feed", "DailyBudget", "Bid", "Currency"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )
	

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of age
        if self.age:
            _dict['Age'] = self.age.to_dict()
        # override the default output from pydantic by calling `to_dict()` of gender
        if self.gender:
            _dict['Gender'] = self.gender.to_dict()
        # override the default output from pydantic by calling `to_dict()` of audience
        if self.audience:
            _dict['Audience'] = self.audience.to_dict()
        # override the default output from pydantic by calling `to_dict()` of company
        if self.company:
            _dict['Company'] = self.company.to_dict()
        # override the default output from pydantic by calling `to_dict()` of industry
        if self.industry:
            _dict['Industry'] = self.industry.to_dict()
        # override the default output from pydantic by calling `to_dict()` of job_function
        if self.job_function:
            _dict['JobFunction'] = self.job_function.to_dict()
        # override the default output from pydantic by calling `to_dict()` of location
        if self.location:
            _dict['Location'] = self.location.to_dict()
        # override the default output from pydantic by calling `to_dict()` of each item in radius_targets (list)
        _items = []
        if self.radius_targets:
            for _item_radius_targets in self.radius_targets:
                if _item_radius_targets:
                    _items.append(_item_radius_targets.to_dict())
            _dict['RadiusTargets'] = _items
        # override the default output from pydantic by calling `to_dict()` of feed
        if self.feed:
            _dict['Feed'] = self.feed.to_dict()
        # set to None if age (nullable) is None
        # and model_fields_set contains the field
        if self.age is None and "age" in self.model_fields_set:
            _dict['Age'] = None

        # set to None if gender (nullable) is None
        # and model_fields_set contains the field
        if self.gender is None and "gender" in self.model_fields_set:
            _dict['Gender'] = None

        # set to None if audience (nullable) is None
        # and model_fields_set contains the field
        if self.audience is None and "audience" in self.model_fields_set:
            _dict['Audience'] = None

        # set to None if company (nullable) is None
        # and model_fields_set contains the field
        if self.company is None and "company" in self.model_fields_set:
            _dict['Company'] = None

        # set to None if industry (nullable) is None
        # and model_fields_set contains the field
        if self.industry is None and "industry" in self.model_fields_set:
            _dict['Industry'] = None

        # set to None if job_function (nullable) is None
        # and model_fields_set contains the field
        if self.job_function is None and "job_function" in self.model_fields_set:
            _dict['JobFunction'] = None

        # set to None if location (nullable) is None
        # and model_fields_set contains the field
        if self.location is None and "location" in self.model_fields_set:
            _dict['Location'] = None

        # set to None if radius_targets (nullable) is None
        # and model_fields_set contains the field
        if self.radius_targets is None and "radius_targets" in self.model_fields_set:
            _dict['RadiusTargets'] = None

        # set to None if campaign_bidding_strategy (nullable) is None
        # and model_fields_set contains the field
        if self.campaign_bidding_strategy is None and "campaign_bidding_strategy" in self.model_fields_set:
            _dict['CampaignBiddingStrategy'] = None

        # set to None if campaign_sub_type (nullable) is None
        # and model_fields_set contains the field
        if self.campaign_sub_type is None and "campaign_sub_type" in self.model_fields_set:
            _dict['CampaignSubType'] = None

        # set to None if feed (nullable) is None
        # and model_fields_set contains the field
        if self.feed is None and "feed" in self.model_fields_set:
            _dict['Feed'] = None

        # set to None if daily_budget (nullable) is None
        # and model_fields_set contains the field
        if self.daily_budget is None and "daily_budget" in self.model_fields_set:
            _dict['DailyBudget'] = None

        # set to None if bid (nullable) is None
        # and model_fields_set contains the field
        if self.bid is None and "bid" in self.model_fields_set:
            _dict['Bid'] = None

        # set to None if currency (nullable) is None
        # and model_fields_set contains the field
        if self.currency is None and "currency" in self.model_fields_set:
            _dict['Currency'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of GetAudienceFullEstimationRequest from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "Age": SelectionOfAgeEnum.from_dict(obj["Age"]) if obj.get("Age") is not None else None,
                        "Gender": SelectionOfGenderEnum.from_dict(obj["Gender"]) if obj.get("Gender") is not None else None,
                        "Audience": SelectionOflong.from_dict(obj["Audience"]) if obj.get("Audience") is not None else None,
                        "Company": SelectionOflong.from_dict(obj["Company"]) if obj.get("Company") is not None else None,
                        "Industry": SelectionOflong.from_dict(obj["Industry"]) if obj.get("Industry") is not None else None,
                        "JobFunction": SelectionOflong.from_dict(obj["JobFunction"]) if obj.get("JobFunction") is not None else None,
                        "Location": SelectionOflong.from_dict(obj["Location"]) if obj.get("Location") is not None else None,
                        "RadiusTargets": [RadiusTarget.from_dict(_item) for _item in obj["RadiusTargets"]] if obj.get("RadiusTargets") is not None else None,
                        "CampaignBiddingStrategy": obj.get("CampaignBiddingStrategy") if obj.get("CampaignBiddingStrategy") is not None else None,
                        "CampaignSubType": obj.get("CampaignSubType") if obj.get("CampaignSubType") is not None else None,
                        "Feed": Feed.from_dict(obj["Feed"]) if obj.get("Feed") is not None else None,
                        "DailyBudget": obj.get("DailyBudget") if obj.get("DailyBudget") is not None else None,
                        "Bid": obj.get("Bid") if obj.get("Bid") is not None else None,
                        "Currency": obj.get("Currency") if obj.get("Currency") is not None else None
        })
        return _obj



from bingads.v13.internal.bulk.object_writer import _BulkObjectWriter


class BulkFileWriter:
    """ Provides methods to write bulk entities to a file.

    For more information about the Bulk File Schema, see https://go.microsoft.com/fwlink/?linkid=846127.

    :param file_path: The file path of the bulk file to write.
    :type file_path: str
    :param file_type: The bulk file type.
    :type file_type: str
    """

    def __init__(self, file_path, file_type='Csv'):
        self._file_path = file_path
        self._file_type = file_type
        self._bulk_object_writer = _BulkObjectWriter(file_path=self.file_path, file_type=self.file_type)
        self._bulk_object_writer.__enter__()
        self._bulk_object_writer.write_file_metadata()

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_value, traceback):
        self._bulk_object_writer.__exit__(exc_type, exc_value, traceback)

    def close(self):
        self.__exit__(None, None, None)

    def write_entity(self, entity, exclude_readonly_data=False):
        """ Writes the specified :class:`.BulkEntity` to the file.

        Bulk entities that are derived from :class:`._SingleRecordBulkEntity` will be written to a single row in the file.
        Bulk entities that are derived from :class:`._MultiRecordBulkEntity` will be written to multiple rows in the file.

        :param entity: The bulk entity to write to the file.
        :type entity: BulkEntity
        :param exclude_readonly_data: excludeReadonlyData indicates whether readonly data (such as errors, performance data etc.)
                                    should be excluded when writing to file
        :type exclude_readonly_data: bool
        :rtype: None
        """

        entity.write_to_stream(self._bulk_object_writer, exclude_readonly_data=exclude_readonly_data)

    @property
    def file_path(self):
        """ The file path of the bulk file to write.

        :rtype: str
        """

        return self._file_path

    @property
    def file_type(self):
        """ The bulk file type.

        :rtype: str
        """

        return self._file_type



# coding: utf-8

"""
    CustomerManagementService

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import json
from enum import Enum
from typing_extensions import Self

class ClientLinkStatus(str, Enum):
    LINKPENDING = 'LinkPending'

    LINKCANCELED = 'LinkCanceled'

    LINKEXPIRED = 'LinkExpired'

    LINKACCEPTED = 'LinkAccepted'

    LINKDECLINED = 'LinkDeclined'

    LINKINPROGRESS = 'LinkInProgress'

    ACTIVE = 'Active'

    LINKFAILED = 'LinkFailed'

    UNLINKREQUESTED = 'UnlinkRequested'

    UNLINKPENDING = 'UnlinkPending'

    UNLINKCANCELED = 'UnlinkCanceled'

    UNLINKINPROGRESS = 'UnlinkInProgress'

    INACTIVE = 'Inactive'

    UNLINKFAILED = 'UnlinkFailed'

	
    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of ClientLinkStatus from a JSON string"""
        return cls(json.loads(json_str))





# coding: utf-8

"""
    ReportingService

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import json
from enum import Enum
from typing_extensions import Self

class ReportTimeZone(str, Enum):
    NUKUALOFA = 'Nukualofa'

    FIJIKAMCHATKAMARSHALLISLAND = 'FijiKamchatkaMarshallIsland'

    AUCKLANDWELLINGTON = 'AucklandWellington'

    MAGADANSOLOMONISLANDNEWCALEDONIA = 'MagadanSolomonIslandNewCaledonia'

    VLADIVOSTOK = 'Vladivostok'

    HOBART = 'Hobart'

    GUAMPORTMORESBY = 'GuamPortMoresby'

    CANBERRAMELBOURNESYDNEY = 'CanberraMelbourneSydney'

    BRISBANE = 'Brisbane'

    DARWIN = 'Darwin'

    ADELAIDE = 'Adelaide'

    YAKUTSK = 'Yakutsk'

    SEOUL = 'Seoul'

    OSAKASAPPOROTOKYO = 'OsakaSapporoTokyo'

    TAIPEI = 'Taipei'

    PERTH = 'Perth'

    KUALALUMPURSINGAPORE = 'KualaLumpurSingapore'

    IRKUTSKULAANBATAAR = 'IrkutskUlaanBataar'

    BEIJINGCHONGQINGHONGKONGURUMQI = 'BeijingChongqingHongKongUrumqi'

    KRASNOYARSK = 'Krasnoyarsk'

    BANGKOKHANOIJAKARTA = 'BangkokHanoiJakarta'

    RANGOON = 'Rangoon'

    SRIJAYAWARDENEPURA = 'SriJayawardenepura'

    ASTANADHAKA = 'AstanaDhaka'

    ALMATYNOVOSIBIRSK = 'AlmatyNovosibirsk'

    KATHMANDU = 'Kathmandu'

    CHENNAIKOLKATAMUMBAINEWDELHI = 'ChennaiKolkataMumbaiNewDelhi'

    ISLANDAMABADKARACHITASHKENT = 'IslandamabadKarachiTashkent'

    EKATERINBURG = 'Ekaterinburg'

    KABUL = 'Kabul'

    BAKUTBILISIYEREVAN = 'BakuTbilisiYerevan'

    ABUDHABIMUSCAT = 'AbuDhabiMuscat'

    TEHRAN = 'Tehran'

    NAIROBI = 'Nairobi'

    MOSCOWSTPETERSBURGVOLGOGRAD = 'MoscowStPetersburgVolgograd'

    KUWAITRIYADH = 'KuwaitRiyadh'

    BAGHDAD = 'Baghdad'

    JERUSALEM = 'Jerusalem'

    HELSINKIKYIVRIGASOFIATALLINNVILNIUS = 'HelsinkiKyivRigaSofiaTallinnVilnius'

    HARAREPRETORIA = 'HararePretoria'

    CAIRO = 'Cairo'

    BUCHAREST = 'Bucharest'

    ATHENSISLANDANBULMINSK = 'AthensIslandanbulMinsk'

    WESTCENTRALAFRICA = 'WestCentralAfrica'

    SARAJEVOSKOPJEWARSAWZAGREB = 'SarajevoSkopjeWarsawZagreb'

    BRUSSELSCOPENHAGENMADRIDPARIS = 'BrusselsCopenhagenMadridParis'

    BELGRADEBRATISLAVABUDAPESTLJUBLJANAPRAGUE = 'BelgradeBratislavaBudapestLjubljanaPrague'

    AMSTERDAMBERLINBERNROMESTOCKHOLMVIENNA = 'AmsterdamBerlinBernRomeStockholmVienna'

    CASABLANCAMONROVIA = 'CasablancaMonrovia'

    GREENWICHMEANTIMEDUBLINEDINBURGHLISBONLONDON = 'GreenwichMeanTimeDublinEdinburghLisbonLondon'

    AZORES = 'Azores'

    CAPEVERDEISLAND = 'CapeVerdeIsland'

    MIDATLANTIC = 'MidAtlantic'

    BRASILIA = 'Brasilia'

    BUENOSAIRESGEORGETOWN = 'BuenosAiresGeorgetown'

    GREENLAND = 'Greenland'

    NEWFOUNDLAND = 'Newfoundland'

    ATLANTICTIMECANADA = 'AtlanticTimeCanada'

    CARACASLAPAZ = 'CaracasLaPaz'

    SANTIAGO = 'Santiago'

    BOGOTALIMAQUITO = 'BogotaLimaQuito'

    EASTERNTIMEUSCANADA = 'EasternTimeUSCanada'

    INDIANAEAST = 'IndianaEast'

    CENTRALAMERICA = 'CentralAmerica'

    CENTRALTIMEUSCANADA = 'CentralTimeUSCanada'

    GUADALAJARAMEXICOCITYMONTERREY = 'GuadalajaraMexicoCityMonterrey'

    SASKATCHEWAN = 'Saskatchewan'

    ARIZONA = 'Arizona'

    CHIHUAHUALAPAZMAZATLAN = 'ChihuahuaLaPazMazatlan'

    MOUNTAINTIMEUSCANADA = 'MountainTimeUSCanada'

    PACIFICTIMEUSCANADATIJUANA = 'PacificTimeUSCanadaTijuana'

    ALASKA = 'Alaska'

    HAWAII = 'Hawaii'

    MIDWAYISLANDANDSAMOA = 'MidwayIslandandSamoa'

    INTERNATIONALDATELINEWEST = 'InternationalDateLineWest'

	
    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of ReportTimeZone from a JSON string"""
        return cls(json.loads(json_str))





from bingads.service_client import _CAMPAIGN_OBJECT_FACTORY_V13
from bingads.v13.internal.bulk.string_table import _StringTable
from bingads.v13.internal.bulk.entities.single_record_bulk_entity import _SingleRecordBulkEntity
from bingads.v13.internal.bulk.mappings import _SimpleBulkMapping
from bingads.v13.internal.extensions import *

# Define type used
ProductAd = type(_CAMPAIGN_OBJECT_FACTORY_V13.create('ProductAd'))
TextAd = type(_CAMPAIGN_OBJECT_FACTORY_V13.create('TextAd'))
AppInstallAd = type(_CAMPAIGN_OBJECT_FACTORY_V13.create('AppInstallAd'))
ExpandedTextAd = type(_CAMPAIGN_OBJECT_FACTORY_V13.create('ExpandedTextAd'))
DynamicSearchAd = type(_CAMPAIGN_OBJECT_FACTORY_V13.create('DynamicSearchAd'))
ResponsiveSearchAd = type(_CAMPAIGN_OBJECT_FACTORY_V13.create('ResponsiveSearchAd'))
ResponsiveAd = type(_CAMPAIGN_OBJECT_FACTORY_V13.create('ResponsiveAd'))

class _BulkAd(_SingleRecordBulkEntity):
    """ This abstract base class provides properties that are shared by all bulk ad classes.

    *See also:*

    * :class:`.BulkProductAd`
    * :class:`.BulkTextAd`
    * :class:`.BulkAppInstallAd`
    * :class:`.BulkExpandedTextAd`
    * :class:`.BulkDynamicSearchAd`
    * :class:`.BulkResponsiveAd`
    * :class:`.BulkResponsiveSearchAd`
    """

    def __init__(self,
                 ad_group_id=None,
                 campaign_name=None,
                 ad_group_name=None,
                 ad=None):
        super(_BulkAd, self).__init__()

        self._ad_group_id = ad_group_id
        self._campaign_name = campaign_name
        self._ad_group_name = ad_group_name
        self._ad = ad
        self._performance_data = None
        self._editorial_appeal_status = None

    @property
    def editorial_appeal_status(self):
        """ The editorial appeal status of the ad.

        :rtype: str
        """

        return self._editorial_appeal_status

    @editorial_appeal_status.setter
    def editorial_appeal_status(self, editorial_appeal_status):
        self._editorial_appeal_status = editorial_appeal_status
        
    @property
    def ad_group_id(self):
        """ The identifier of the ad group that contains the ad.

        Corresponds to the 'Parent Id' field in the bulk file.

        :rtype: int
        """

        return self._ad_group_id

    @ad_group_id.setter
    def ad_group_id(self, ad_group_id):
        self._ad_group_id = ad_group_id

    @property
    def campaign_name(self):
        """ The name of the campaign that contains the ad.

        Corresponds to the 'Campaign' field in the bulk file.

        :rtype: str
        """

        return self._campaign_name

    @campaign_name.setter
    def campaign_name(self, campaign_name):
        self._campaign_name = campaign_name

    @property
    def ad_group_name(self):
        """ The name of the ad group that contains the ad.

        Corresponds to the 'Ad Group' field in the bulk file.

        :rtype: str
        """

        return self._ad_group_name

    @ad_group_name.setter
    def ad_group_name(self, ad_group_name):
        self._ad_group_name = ad_group_name

    @property
    def ad(self):
        """ The type of ad.

        """

        return self._ad

    @ad.setter
    def ad(self, ad):
        self._ad = ad

    _MAPPINGS = [
        _SimpleBulkMapping(
            header=_StringTable.Status,
            field_to_csv=lambda c: bulk_str(c.ad.Status),
            csv_to_field=lambda c, v: csv_to_field_enum(c.ad, v, 'Status', AdStatus)
        ),
        _SimpleBulkMapping(
            header=_StringTable.Id,
            field_to_csv=lambda c: bulk_str(c.ad.Id),
            csv_to_field=lambda c, v: setattr(c.ad, 'Id', int(v) if v else None)
        ),
        _SimpleBulkMapping(
            header=_StringTable.ParentId,
            field_to_csv=lambda c: bulk_str(c.ad_group_id),
            csv_to_field=lambda c, v: setattr(c, '_ad_group_id', int(v) if v else None)
        ),
        _SimpleBulkMapping(
            header=_StringTable.Campaign,
            field_to_csv=lambda c: c.campaign_name,
            csv_to_field=lambda c, v: setattr(c, '_campaign_name', v)
        ),
        _SimpleBulkMapping(
            header=_StringTable.AdGroup,
            field_to_csv=lambda c: c.ad_group_name,
            csv_to_field=lambda c, v: setattr(c, '_ad_group_name', v)
        ),
        _SimpleBulkMapping(
            header=_StringTable.EditorialStatus,
            field_to_csv=lambda c: c.ad.EditorialStatus,
            csv_to_field=lambda c, v: csv_to_field_enum(c.ad, v, 'EditorialStatus', AdEditorialStatus)
        ),
        _SimpleBulkMapping(
            header=_StringTable.DevicePreference,
            field_to_csv=lambda c: bulk_device_preference_str(c.ad.DevicePreference),
            csv_to_field=lambda c, v: setattr(c.ad, 'DevicePreference', parse_device_preference(v))
        ),

        _SimpleBulkMapping(
            header=_StringTable.AdFormatPreference,
            field_to_csv=lambda c: bulk_str(c.ad.AdFormatPreference),
            csv_to_field=lambda c, v: setattr(c.ad, 'AdFormatPreference', v if v else None)
        ),
        _SimpleBulkMapping(
            header=_StringTable.FinalUrl,
            field_to_csv=lambda c: field_to_csv_Urls(c.ad.FinalUrls, c.ad.Id),
            csv_to_field=lambda c, v: csv_to_field_Urls(c.ad.FinalUrls, v)
        ),
        _SimpleBulkMapping(
            header=_StringTable.FinalMobileUrl,
            field_to_csv=lambda c: field_to_csv_Urls(c.ad.FinalMobileUrls, c.ad.Id),
            csv_to_field=lambda c, v: csv_to_field_Urls(c.ad.FinalMobileUrls, v)
        ),
        _SimpleBulkMapping(
            header=_StringTable.TrackingTemplate,
            field_to_csv=lambda c: bulk_str(c.ad.TrackingUrlTemplate),
            csv_to_field=lambda c, v: setattr(c.ad, 'TrackingUrlTemplate', v if v else None)
        ),
        _SimpleBulkMapping(
            header=_StringTable.CustomParameter,
            field_to_csv=lambda c: field_to_csv_UrlCustomParameters(c.ad),
            csv_to_field=lambda c, v: csv_to_field_UrlCustomParameters(c.ad, v)
        ),
        _SimpleBulkMapping(
            header=_StringTable.FinalUrlSuffix,
            field_to_csv=lambda c: bulk_optional_str(c.ad.FinalUrlSuffix, c.ad.Id),
            csv_to_field=lambda c, v: setattr(c.ad, 'FinalUrlSuffix', v if v else None)
        ),
        _SimpleBulkMapping(
            header=_StringTable.EditorialAppealStatus,
            field_to_csv=lambda c: c.editorial_appeal_status,
            csv_to_field=lambda c, v: setattr(c, '_editorial_appeal_status', v)
        )
    ]

    def process_mappings_to_row_values(self, row_values, exclude_readonly_data):
        self.convert_to_values(row_values, _BulkAd._MAPPINGS)

    def process_mappings_from_row_values(self, row_values):
        row_values.convert_to_entity(self, _BulkAd._MAPPINGS)

    def read_additional_data(self, stream_reader):
        super(_BulkAd, self).read_additional_data(stream_reader)


class BulkProductAd(_BulkAd):
    """ Represents a product ad.

    This class exposes the :attr:`product_ad` property that can be read and written as fields of the Product Ad record in a bulk file.

    For more information, see Product Ad at https://go.microsoft.com/fwlink/?linkid=846127.

    *See also:*

    * :class:`.BulkServiceManager`
    * :class:`.BulkOperation`
    * :class:`.BulkFileReader`
    * :class:`.BulkFileWriter`
    """

    def __init__(self,
                 ad_group_id=None,
                 campaign_name=None,
                 ad_group_name=None,
                 ad=None):
        super(BulkProductAd, self).__init__(
            ad_group_id,
            campaign_name,
            ad_group_name,
            ad
        )
        self.product_ad = ad

    @property
    def product_ad(self):
        """ The product ad.

        See Product Ad at: https://go.microsoft.com/fwlink/?linkid=846127.
        """

        return self._ad

    @product_ad.setter
    def product_ad(self, product_ad):
        if product_ad is not None and not isinstance(product_ad, ProductAd):
            raise ValueError('Not an instance of ProductAd')
        self._ad = product_ad

    _MAPPINGS = [
        _SimpleBulkMapping(
            header=_StringTable.PromotionalText,
            field_to_csv=lambda c: bulk_optional_str(c.product_ad.PromotionalText, c.product_ad.Id),
            csv_to_field=lambda c, v: setattr(c.product_ad, 'PromotionalText', v if v else '')
        ),
    ]

    def process_mappings_from_row_values(self, row_values):
        self.product_ad = _CAMPAIGN_OBJECT_FACTORY_V13.create('ProductAd')
        self.product_ad.Type = 'Product'
        super(BulkProductAd, self).process_mappings_from_row_values(row_values)
        row_values.convert_to_entity(self, BulkProductAd._MAPPINGS)

    def process_mappings_to_row_values(self, row_values, exclude_readonly_data):
        self._validate_property_not_null(self.product_ad, 'product_ad')
        super(BulkProductAd, self).process_mappings_to_row_values(row_values, exclude_readonly_data)
        self.convert_to_values(row_values, BulkProductAd._MAPPINGS)


class BulkTextAd(_BulkAd):
    """ Represents a Text Ad.

    This class exposes the :attr:`text_ad` property that can be read and written as fields of the Text Ad record in a bulk file.

    For more information, see Text Ad at https://go.microsoft.com/fwlink/?linkid=846127.

    *See also:*

    * :class:`.BulkServiceManager`
    * :class:`.BulkOperation`
    * :class:`.BulkFileReader`
    * :class:`.BulkFileWriter`
    """

    def __init__(self,
                 ad_group_id=None,
                 campaign_name=None,
                 ad_group_name=None,
                 ad=None):
        super(BulkTextAd, self).__init__(
            ad_group_id,
            campaign_name,
            ad_group_name,
            ad,
        )
        self.text_ad = ad

    @property
    def text_ad(self):
        """ The text ad.

        see Text Ad at https://go.microsoft.com/fwlink/?linkid=846127.
        """

        return self._ad

    @text_ad.setter
    def text_ad(self, text_ad):
        if text_ad is not None and not isinstance(text_ad, TextAd):
            raise ValueError('Not an instance of TextAd')
        self._ad = text_ad

    _MAPPINGS = [
        _SimpleBulkMapping(
            header=_StringTable.Title,
            field_to_csv=lambda c: c.text_ad.Title,
            csv_to_field=lambda c, v: setattr(c.text_ad, 'Title', v)
        ),
        _SimpleBulkMapping(
            header=_StringTable.Text,
            field_to_csv=lambda c: c.text_ad.Text,
            csv_to_field=lambda c, v: setattr(c.text_ad, 'Text', v)
        ),
        _SimpleBulkMapping(
            header=_StringTable.DisplayUrl,
            field_to_csv=lambda c: bulk_optional_str(c.text_ad.DisplayUrl, c.text_ad.Id),
            csv_to_field=lambda c, v: setattr(c.text_ad, 'DisplayUrl', v if v else '')
        ),
        _SimpleBulkMapping(
            header=_StringTable.DestinationUrl,
            field_to_csv=lambda c: bulk_optional_str(c.text_ad.DestinationUrl, c.text_ad.Id),
            csv_to_field=lambda c, v: setattr(c.text_ad, 'DestinationUrl', v if v else '')
        ),
    ]

    def process_mappings_from_row_values(self, row_values):
        self.text_ad = _CAMPAIGN_OBJECT_FACTORY_V13.create('TextAd')
        self.text_ad.Type = 'Text'
        super(BulkTextAd, self).process_mappings_from_row_values(row_values)
        row_values.convert_to_entity(self, BulkTextAd._MAPPINGS)

    def process_mappings_to_row_values(self, row_values, exclude_readonly_data):
        self._validate_property_not_null(self.text_ad, 'text_ad')
        super(BulkTextAd, self).process_mappings_to_row_values(row_values, exclude_readonly_data)
        self.convert_to_values(row_values, BulkTextAd._MAPPINGS)


class BulkAppInstallAd(_BulkAd):
    """ Represents an App Install Ad.

    This class exposes the :attr:`app_install_ad` property that can be read and written as fields of the App Install Ad record in a bulk file.

    For more information, see App Install Ad at https://go.microsoft.com/fwlink/?linkid=846127.

    *See also:*

    * :class:`.BulkServiceManager`
    * :class:`.BulkOperation`
    * :class:`.BulkFileReader`
    * :class:`.BulkFileWriter`
    """

    def __init__(self,
                 ad_group_id=None,
                 campaign_name=None,
                 ad_group_name=None,
                 ad=None):
        super(BulkAppInstallAd, self).__init__(
            ad_group_id,
            campaign_name,
            ad_group_name,
            ad,
        )
        self.app_install_ad = ad

    @property
    def app_install_ad(self):
        """ The App Install Ad.

        see App Install Ad at https://go.microsoft.com/fwlink/?linkid=846127.
        """

        return self._ad

    @app_install_ad.setter
    def app_install_ad(self, app_install_ad):
        if app_install_ad is not None and not isinstance(app_install_ad, AppInstallAd):
            raise ValueError('Not an instance of AppInstallAd')
        self._ad = app_install_ad

    _MAPPINGS = [
        _SimpleBulkMapping(
            header=_StringTable.AppPlatform,
            field_to_csv=lambda c: c.app_install_ad.AppPlatform,
            csv_to_field=lambda c, v: setattr(c.app_install_ad, 'AppPlatform', v)
        ),
        _SimpleBulkMapping(
            header=_StringTable.AppStoreId,
            field_to_csv=lambda c: c.app_install_ad.AppStoreId,
            csv_to_field=lambda c, v: setattr(c.app_install_ad, 'AppStoreId', v)
        ),
        _SimpleBulkMapping(
            header=_StringTable.Title,
            field_to_csv=lambda c: c.app_install_ad.Title,
            csv_to_field=lambda c, v: setattr(c.app_install_ad, 'Title', v)
        ),
        _SimpleBulkMapping(
            header=_StringTable.Text,
            field_to_csv=lambda c: c.app_install_ad.Text,
            csv_to_field=lambda c, v: setattr(c.app_install_ad, 'Text', v)
        ),
    ]

    def process_mappings_from_row_values(self, row_values):
        self.app_install_ad = _CAMPAIGN_OBJECT_FACTORY_V13.create('AppInstallAd')
        self.app_install_ad.Type = 'AppInstall'
        super(BulkAppInstallAd, self).process_mappings_from_row_values(row_values)
        row_values.convert_to_entity(self, BulkAppInstallAd._MAPPINGS)

    def process_mappings_to_row_values(self, row_values, exclude_readonly_data):
        self._validate_property_not_null(self.app_install_ad, 'app_install_ad')
        super(BulkAppInstallAd, self).process_mappings_to_row_values(row_values, exclude_readonly_data)
        self.convert_to_values(row_values, BulkAppInstallAd._MAPPINGS)


class BulkExpandedTextAd(_BulkAd):
    """ Represents an Expanded Text Ad.

    This class exposes the :attr:`expanded_text_ad` property that can be read and written as fields of the Expanded Text Ad record in a bulk file.

    For more information, see Expanded Text Ad at https://go.microsoft.com/fwlink/?linkid=846127.

    *See also:*

    * :class:`.BulkServiceManager`
    * :class:`.BulkOperation`
    * :class:`.BulkFileReader`
    * :class:`.BulkFileWriter`
    """

    def __init__(self,
                 ad_group_id=None,
                 campaign_name=None,
                 ad_group_name=None,
                 ad=None):
        super(BulkExpandedTextAd, self).__init__(
            ad_group_id,
            campaign_name,
            ad_group_name,
            ad,
        )
        self.expanded_text_ad = ad

    @property
    def expanded_text_ad(self):
        """ The Expanded Text Ad.

        see Expanded Text Ad at https://go.microsoft.com/fwlink/?linkid=846127.
        """

        return self._ad

    @expanded_text_ad.setter
    def expanded_text_ad(self, expanded_text_ad):
        if expanded_text_ad is not None and not isinstance(expanded_text_ad, ExpandedTextAd):
            raise ValueError('Not an instance of ExpandedTextAd')
        self._ad = expanded_text_ad

    _MAPPINGS = [
        _SimpleBulkMapping(
            header=_StringTable.Text,
            field_to_csv=lambda c: c.expanded_text_ad.Text,
            csv_to_field=lambda c, v: setattr(c.expanded_text_ad, 'Text', v)
        ),
        _SimpleBulkMapping(
            header=_StringTable.TextPart2,
            field_to_csv=lambda c: bulk_optional_str(c.expanded_text_ad.TextPart2, c.expanded_text_ad.Id),
            csv_to_field=lambda c, v: setattr(c.expanded_text_ad, 'TextPart2', v if v else '')
        ),
        _SimpleBulkMapping(
            header=_StringTable.TitlePart1,
            field_to_csv=lambda c: c.expanded_text_ad.TitlePart1,
            csv_to_field=lambda c, v: setattr(c.expanded_text_ad, 'TitlePart1', v)
        ),
        _SimpleBulkMapping(
            header=_StringTable.TitlePart2,
            field_to_csv=lambda c: c.expanded_text_ad.TitlePart2,
            csv_to_field=lambda c, v: setattr(c.expanded_text_ad, 'TitlePart2', v)
        ),
        _SimpleBulkMapping(
            header=_StringTable.TitlePart3,
            field_to_csv=lambda c: bulk_optional_str(c.expanded_text_ad.TitlePart3, c.expanded_text_ad.Id),
            csv_to_field=lambda c, v: setattr(c.expanded_text_ad, 'TitlePart3', v if v else '')
        ),
        _SimpleBulkMapping(
            header=_StringTable.Path1,
            field_to_csv=lambda c: bulk_optional_str(c.expanded_text_ad.Path1, c.expanded_text_ad.Id),
            csv_to_field=lambda c, v: setattr(c.expanded_text_ad, 'Path1', v)
        ),
        _SimpleBulkMapping(
            header=_StringTable.Path2,
            field_to_csv=lambda c: bulk_optional_str(c.expanded_text_ad.Path2, c.expanded_text_ad.Id),
            csv_to_field=lambda c, v: setattr(c.expanded_text_ad, 'Path2', v)
        ),
        _SimpleBulkMapping(
            header=_StringTable.Domain,
            field_to_csv=lambda c: bulk_optional_str(c.expanded_text_ad.Domain, c.expanded_text_ad.Id),
            csv_to_field=lambda c, v: setattr(c.expanded_text_ad, 'Domain', v)
        ),
    ]

    def process_mappings_from_row_values(self, row_values):
        self.expanded_text_ad = _CAMPAIGN_OBJECT_FACTORY_V13.create('ExpandedTextAd')
        self.expanded_text_ad.Type = 'ExpandedText'
        super(BulkExpandedTextAd, self).process_mappings_from_row_values(row_values)
        row_values.convert_to_entity(self, BulkExpandedTextAd._MAPPINGS)

    def process_mappings_to_row_values(self, row_values, exclude_readonly_data):
        self._validate_property_not_null(self.expanded_text_ad, 'expanded_text_ad')
        super(BulkExpandedTextAd, self).process_mappings_to_row_values(row_values, exclude_readonly_data)
        self.convert_to_values(row_values, BulkExpandedTextAd._MAPPINGS)


class BulkDynamicSearchAd(_BulkAd):
    """ Represents a Dynamic Search Ad.

    This class exposes the :attr:`dynamic_search_ad` property that can be read and written as fields of the Dynamic Search Ad record in a bulk file.

    For more information, see Dynamic Search Ad at https://go.microsoft.com/fwlink/?linkid=836840.

    *See also:*

    * :class:`.BulkServiceManager`
    * :class:`.BulkOperation`
    * :class:`.BulkFileReader`
    * :class:`.BulkFileWriter`
    """

    def __init__(self,
                 ad_group_id=None,
                 campaign_name=None,
                 ad_group_name=None,
                 ad=None):
        super(BulkDynamicSearchAd, self).__init__(
            ad_group_id,
            campaign_name,
            ad_group_name,
            ad,
        )
        self.dynamic_search_ad = ad

    @property
    def dynamic_search_ad(self):
        """ The dynamic search ad.

        see Dynamic Search Ad at https://go.microsoft.com/fwlink/?linkid=836840.
        """

        return self._ad

    @dynamic_search_ad.setter
    def dynamic_search_ad(self, dynamic_search_ad):
        if dynamic_search_ad is not None and not isinstance(dynamic_search_ad, DynamicSearchAd):
            raise ValueError('Not an instance of DynamicSearchAd')
        self._ad = dynamic_search_ad

    _MAPPINGS = [
        _SimpleBulkMapping(
            header=_StringTable.Text,
            field_to_csv=lambda c: c.dynamic_search_ad.Text,
            csv_to_field=lambda c, v: setattr(c.dynamic_search_ad, 'Text', v)
        ),
        _SimpleBulkMapping(
            header=_StringTable.Path1,
            field_to_csv=lambda c: c.dynamic_search_ad.Path1,
            csv_to_field=lambda c, v: setattr(c.dynamic_search_ad, 'Path1', v)
        ),
        _SimpleBulkMapping(
            header=_StringTable.Path2,
            field_to_csv=lambda c: c.dynamic_search_ad.Path2,
            csv_to_field=lambda c, v: setattr(c.dynamic_search_ad, 'Path2', v)
        ),
        _SimpleBulkMapping(
            header=_StringTable.TextPart2,
            field_to_csv=lambda c: c.dynamic_search_ad.TextPart2,
            csv_to_field=lambda c, v: setattr(c.dynamic_search_ad, 'TextPart2', v)
        ),
    ]

    def process_mappings_from_row_values(self, row_values):
        self.dynamic_search_ad = _CAMPAIGN_OBJECT_FACTORY_V13.create('DynamicSearchAd')
        self.dynamic_search_ad.Type = 'DynamicSearch'
        super(BulkDynamicSearchAd, self).process_mappings_from_row_values(row_values)
        row_values.convert_to_entity(self, BulkDynamicSearchAd._MAPPINGS)

    def process_mappings_to_row_values(self, row_values, exclude_readonly_data):
        self._validate_property_not_null(self.dynamic_search_ad, 'dynamic_search_ad')
        super(BulkDynamicSearchAd, self).process_mappings_to_row_values(row_values, exclude_readonly_data)
        self.convert_to_values(row_values, BulkDynamicSearchAd._MAPPINGS)


class BulkResponsiveAd(_BulkAd):
    """ Represents a Responsive Ad.

    This class exposes the :attr:`responsive_ad` property that can be read and written as fields of the Responsive Ad record in a bulk file.

    For more information, see Responsive Ad at https://go.microsoft.com/fwlink/?linkid=836840.

    *See also:*

    * :class:`.BulkServiceManager`
    * :class:`.BulkOperation`
    * :class:`.BulkFileReader`
    * :class:`.BulkFileWriter`
    """

    def __init__(self,
                 ad_group_id=None,
                 campaign_name=None,
                 ad_group_name=None,
                 ad=None):
        super(BulkResponsiveAd, self).__init__(
            ad_group_id,
            campaign_name,
            ad_group_name,
            ad,
        )
        self._ad = ad
        self._verified_tracking_data = None

    @property
    def responsive_ad(self):
        """ The responsive search ad.

        see Responsive Ad at https://go.microsoft.com/fwlink/?linkid=836840.
        """

        return self._ad

    @responsive_ad.setter
    def responsive_ad(self, responsive_ad):
        if responsive_ad is not None and not isinstance(responsive_ad, ResponsiveAd):
            raise ValueError('Not an instance of ResponsiveAd')
        self._ad = responsive_ad    
    
    @property
    def verified_tracking_data(self):
        """
        The verified tracking data that the ad associated

        Corresponds to 'Verified Tracking Setting' field in bulk file.
        :rtype: ArrayOfArrayOfKeyValuePairOfstringstring
        """
        return self._verified_tracking_data

    @verified_tracking_data.setter
    def verified_tracking_data(self, value):
        self._verified_tracking_data = value

    _MAPPINGS = [
        _SimpleBulkMapping(
            header=_StringTable.BusinessName,
            field_to_csv=lambda c: c.responsive_ad.BusinessName,
            csv_to_field=lambda c, v: setattr(c.responsive_ad, 'BusinessName', v)
        ),
        _SimpleBulkMapping(
            header=_StringTable.CallToAction,
            field_to_csv=lambda c: c.responsive_ad.CallToAction,
            csv_to_field=lambda c, v: setattr(c.responsive_ad, 'CallToAction', v if v else None)
        ),
        _SimpleBulkMapping(
            header=_StringTable.Headline,
            field_to_csv=lambda c: c.responsive_ad.Headline,
            csv_to_field=lambda c, v: setattr(c.responsive_ad, 'Headline', v)
        ),
        _SimpleBulkMapping(
            header=_StringTable.LongHeadline,
            field_to_csv=lambda c: c.responsive_ad.LongHeadlineString,
            csv_to_field=lambda c, v: setattr(c.responsive_ad, 'LongHeadlineString', v)
        ),
        _SimpleBulkMapping(
            header=_StringTable.Text,
            field_to_csv=lambda c: c.responsive_ad.Text,
            csv_to_field=lambda c, v: setattr(c.responsive_ad, 'Text', v)
        ),
        _SimpleBulkMapping(
            header=_StringTable.Images,
            field_to_csv=lambda c: field_to_csv_ImageAssetLinks(c.responsive_ad.Images),
            csv_to_field=lambda c, v: csv_to_field_ImageAssetLinks(c.responsive_ad.Images, v)
        ),
        _SimpleBulkMapping(
            header=_StringTable.ImpressionTrackingUrls,            
            field_to_csv=lambda c: field_to_csv_Urls(c.responsive_ad.ImpressionTrackingUrls, c.ad.Id),
            csv_to_field=lambda c, v: csv_to_field_Urls(c.responsive_ad.ImpressionTrackingUrls, v)
        ),
        _SimpleBulkMapping(
            header=_StringTable.Descriptions,
            field_to_csv=lambda c: field_to_csv_TextAssetLinks(c.responsive_ad.Descriptions),
            csv_to_field=lambda c, v: csv_to_field_TextAssetLinks(c.responsive_ad.Descriptions ,v)
        ),
        _SimpleBulkMapping(
            header=_StringTable.Headlines,
            field_to_csv=lambda c: field_to_csv_TextAssetLinks(c.responsive_ad.Headlines),
            csv_to_field=lambda c, v: csv_to_field_TextAssetLinks(c.responsive_ad.Headlines, v)
        ),
        _SimpleBulkMapping(
            header=_StringTable.Videos,
            field_to_csv=lambda c: field_to_csv_VideoAssetLinks(c.responsive_ad.Videos),
            csv_to_field=lambda c, v: csv_to_field_VideoAssetLinks(c.responsive_ad.Videos, v)
        ),
        _SimpleBulkMapping(
            header=_StringTable.CallToActionLanguage,
            field_to_csv=lambda c: c.responsive_ad.CallToActionLanguage,
            csv_to_field=lambda c, v: setattr(c.responsive_ad, 'CallToActionLanguage', v if v else None)
        ),
        _SimpleBulkMapping(
            header=_StringTable.LongHeadlines,
            field_to_csv=lambda c: field_to_csv_TextAssetLinks(c.responsive_ad.LongHeadlines),
            csv_to_field=lambda c, v: csv_to_field_TextAssetLinks(c.responsive_ad.LongHeadlines ,v)
        ),
        _SimpleBulkMapping(
            header=_StringTable.Details,
            field_to_csv=lambda c: to_verified_tracking_setting_string(c.verified_tracking_data),
            csv_to_field=lambda c, v: setattr(c, 'verified_tracking_data', parse_verified_tracking_setting(v) if v else None)
        ),
    ]

    def process_mappings_from_row_values(self, row_values):
        self.responsive_ad = _CAMPAIGN_OBJECT_FACTORY_V13.create('ResponsiveAd')
        self.responsive_ad.Type = 'Responsive'
        super(BulkResponsiveAd, self).process_mappings_from_row_values(row_values)
        row_values.convert_to_entity(self, BulkResponsiveAd._MAPPINGS)

    def process_mappings_to_row_values(self, row_values, exclude_readonly_data):
        self._validate_property_not_null(self.responsive_ad, 'responsive_ad')
        super(BulkResponsiveAd, self).process_mappings_to_row_values(row_values, exclude_readonly_data)
        self.convert_to_values(row_values, BulkResponsiveAd._MAPPINGS)

class BulkResponsiveSearchAd(_BulkAd):
    """ Represents a Responsive Search Ad.

    This class exposes the :attr:`responsive_search_ad` property that can be read and written as fields of the Responsive Search Ad record in a bulk file.

    For more information, see Responsive Search Ad at https://go.microsoft.com/fwlink/?linkid=836840.

    *See also:*

    * :class:`.BulkServiceManager`
    * :class:`.BulkOperation`
    * :class:`.BulkFileReader`
    * :class:`.BulkFileWriter`
    """

    def __init__(self,
                 ad_group_id=None,
                 campaign_name=None,
                 ad_group_name=None,
                 ad=None):
        super(BulkResponsiveSearchAd, self).__init__(
            ad_group_id,
            campaign_name,
            ad_group_name,
            ad,
        )
        self._ad = ad

    @property
    def responsive_search_ad(self):
        """ The responsive search ad.

        see Responsive Search Ad at https://go.microsoft.com/fwlink/?linkid=836840.
        """

        return self._ad

    @responsive_search_ad.setter
    def responsive_search_ad(self, rsa):
        if rsa is not None and not isinstance(rsa, ResponsiveSearchAd):
            raise ValueError('Not an instance of ResponsiveSearchAd')
        self._ad = rsa

    _MAPPINGS = [
        _SimpleBulkMapping(
            header=_StringTable.Path1,
            field_to_csv=lambda c: bulk_optional_str(c.responsive_search_ad.Path1, c.responsive_search_ad.Id),
            csv_to_field=lambda c, v: setattr(c.responsive_search_ad, 'Path1', v)
        ),
        _SimpleBulkMapping(
            header=_StringTable.Path2,
            field_to_csv=lambda c: bulk_optional_str(c.responsive_search_ad.Path2, c.responsive_search_ad.Id),
            csv_to_field=lambda c, v: setattr(c.responsive_search_ad, 'Path2', v)
        ),
        _SimpleBulkMapping(
            header=_StringTable.Domain,
            field_to_csv=lambda c: bulk_optional_str(c.responsive_search_ad.Domain, c.responsive_search_ad.Id),
            csv_to_field=lambda c, v: setattr(c.responsive_search_ad, 'Domain', v)
        ),
        _SimpleBulkMapping(
            header=_StringTable.Headline,
            field_to_csv=lambda c: field_to_csv_TextAssetLinks(c.responsive_search_ad.Headlines),
            csv_to_field=lambda c, v: csv_to_field_TextAssetLinks(c.responsive_search_ad.Headlines, v)
        ),
        _SimpleBulkMapping(
            header=_StringTable.Description,
            field_to_csv=lambda c: field_to_csv_TextAssetLinks(c.responsive_search_ad.Descriptions),
            csv_to_field=lambda c, v: csv_to_field_TextAssetLinks(c.responsive_search_ad.Descriptions ,v)
        )
    ]

    def process_mappings_from_row_values(self, row_values):
        self.responsive_search_ad = _CAMPAIGN_OBJECT_FACTORY_V13.create('ResponsiveSearchAd')
        self.responsive_search_ad.Type = 'ResponsiveSearch'
        super(BulkResponsiveSearchAd, self).process_mappings_from_row_values(row_values)
        row_values.convert_to_entity(self, BulkResponsiveSearchAd._MAPPINGS)

    def process_mappings_to_row_values(self, row_values, exclude_readonly_data):
        self._validate_property_not_null(self.responsive_search_ad, 'responsive_search_ad')
        super(BulkResponsiveSearchAd, self).process_mappings_to_row_values(row_values, exclude_readonly_data)
        self.convert_to_values(row_values, BulkResponsiveSearchAd._MAPPINGS)


from bingads.v13.bulk.entities.audiences.bulk_ad_group_negative_audience_association import *

class BulkAdGroupNegativeProductAudienceAssociation(BulkAdGroupNegativeAudienceAssociation):
    """ Represents an Ad Group Negative Product Audience Association that can be read or written in a bulk file.

    For more information, see Ad Group Negative Product Audience Association at https://go.microsoft.com/fwlink/?linkid=846127.

    *See also:*

    * :class:`.BulkServiceManager`
    * :class:`.BulkOperation`
    * :class:`.BulkFileReader`
    * :class:`.BulkFileWriter`
    """



# coding: utf-8

"""
    CampaignManagementService

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import json
from enum import Enum
from typing_extensions import Self

class AdExtensionHeaderType(str, Enum):
    UNKNOWN = 'Unknown'

    AMENITIES = 'Amenities'

    BRANDS = 'Brands'

    CLASSES = 'Classes'

    COURSES = 'Courses'

    DAILYRATES = 'DailyRates'

    DEGREEPROGRAMS = 'DegreePrograms'

    DEPARTMENTS = 'Departments'

    DESTINATIONS = 'Destinations'

    FEATUREDHOTELS = 'FeaturedHotels'

    GOODS = 'Goods'

    GRADES = 'Grades'

    HIGHLIGHTS = 'Highlights'

    INSURANCECOVERAGE = 'InsuranceCoverage'

    ITEMS = 'Items'

    LANGUAGES = 'Languages'

    LOCATIONS = 'Locations'

    MODELS = 'Models'

    NEIGHBORHOODS = 'Neighborhoods'

    PRICES = 'Prices'

    RATES = 'Rates'

    RATINGS = 'Ratings'

    SCHOOLDISTRICTS = 'SchoolDistricts'

    SERVICES = 'Services'

    SERVICECATALOG = 'ServiceCatalog'

    SHOWS = 'Shows'

    SIZES = 'Sizes'

    STYLES = 'Styles'

    TOOLS = 'Tools'

    TOPICS = 'Topics'

    TYPES = 'Types'

    VACATIONS = 'Vacations'

    VEHICLES = 'Vehicles'

    WHAT = 'What'

    WHO = 'Who'

    WHY = 'Why'

    DEALS = 'Deals'

    BESTSELLERS = 'BestSellers'

    AGEGROUPS = 'AgeGroups'

    OCCASIONS = 'Occasions'

    FLOWERS = 'Flowers'

	
    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of AdExtensionHeaderType from a JSON string"""
        return cls(json.loads(json_str))





# coding: utf-8

"""
    CampaignManagementService

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import datetime
from pydantic import BaseModel, ConfigDict, Field, StrictBool, StrictFloat, StrictInt, StrictStr
from typing import Any, ClassVar, Dict, List, Optional, Union, Set
from openapi_client.models.campaign.ad_recommendation_image_suggestion_metadata import AdRecommendationImageSuggestionMetadata
from openapi_client.models.campaign.asset_link import AssetLink
from openapi_client.models.campaign.image import Image
from openapi_client.models.campaign.recommendation_template import RecommendationTemplate
from typing_extensions import Self

class AdRecommendationImageSuggestion(BaseModel):
    """
    AdRecommendationImageSuggestion
    """ # noqa: E501
    image: Optional[Image] = Field(default=None, alias="Image")
    image_url: Optional[StrictStr] = Field(default=None, alias="ImageUrl")
    asset_link: Optional[AssetLink] = Field(default=None, alias="AssetLink")
    image_metadata: Optional[AdRecommendationImageSuggestionMetadata] = Field(default=None, alias="ImageMetadata")
    internal_template: Optional[RecommendationTemplate] = Field(default=None, alias="Internal_Template")
    __properties: ClassVar[List[str]] = ["Image", "ImageUrl", "AssetLink", "ImageMetadata", "Internal_Template"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )
	

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of image
        if self.image:
            _dict['Image'] = self.image.to_dict()
        # override the default output from pydantic by calling `to_dict()` of asset_link
        if self.asset_link:
            _dict['AssetLink'] = self.asset_link.to_dict()
        # override the default output from pydantic by calling `to_dict()` of image_metadata
        if self.image_metadata:
            _dict['ImageMetadata'] = self.image_metadata.to_dict()
        # override the default output from pydantic by calling `to_dict()` of internal_template
        if self.internal_template:
            _dict['Internal_Template'] = self.internal_template.to_dict()
        # set to None if image (nullable) is None
        # and model_fields_set contains the field
        if self.image is None and "image" in self.model_fields_set:
            _dict['Image'] = None

        # set to None if image_url (nullable) is None
        # and model_fields_set contains the field
        if self.image_url is None and "image_url" in self.model_fields_set:
            _dict['ImageUrl'] = None

        # set to None if asset_link (nullable) is None
        # and model_fields_set contains the field
        if self.asset_link is None and "asset_link" in self.model_fields_set:
            _dict['AssetLink'] = None

        # set to None if image_metadata (nullable) is None
        # and model_fields_set contains the field
        if self.image_metadata is None and "image_metadata" in self.model_fields_set:
            _dict['ImageMetadata'] = None

        # set to None if internal_template (nullable) is None
        # and model_fields_set contains the field
        if self.internal_template is None and "internal_template" in self.model_fields_set:
            _dict['Internal_Template'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of AdRecommendationImageSuggestion from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "Image": Image.from_dict(obj["Image"]) if obj.get("Image") is not None else None,
                        "ImageUrl": obj.get("ImageUrl") if obj.get("ImageUrl") is not None else None,
                        "AssetLink": AssetLink.from_dict(obj["AssetLink"]) if obj.get("AssetLink") is not None else None,
                        "ImageMetadata": AdRecommendationImageSuggestionMetadata.from_dict(obj["ImageMetadata"]) if obj.get("ImageMetadata") is not None else None,
                        "Internal_Template": RecommendationTemplate.from_dict(obj["InternalTemplate"]) if obj.get("InternalTemplate") is not None else None
        })
        return _obj



from bingads.v13.internal.bulk.mappings import _SimpleBulkMapping
from bingads.v13.internal.bulk.string_table import _StringTable
from bingads.service_client import _CAMPAIGN_OBJECT_FACTORY_V13

from bingads.v13.internal.extensions import *
from .common import _BulkAdExtensionBase
from .common import _BulkCampaignAdExtensionAssociation
from .common import _BulkAdGroupAdExtensionAssociation
from .common import _BulkAccountAdExtensionAssociation

_VideoAdExtension = type(_CAMPAIGN_OBJECT_FACTORY_V13.create('VideoAdExtension'))


class BulkVideoAdExtension(_BulkAdExtensionBase):
    """ Represents a video ad extension.

    This class exposes the :attr:`video_ad_extension` property that can be read and written
    as fields of the Video Ad Extension record in a bulk file.

    For more information, see Video Ad Extension at https://go.microsoft.com/fwlink/?linkid=846127.

    *See also:*

    * :class:`.BulkServiceManager`
    * :class:`.BulkOperation`
    * :class:`.BulkFileReader`
    * :class:`.BulkFileWriter`
    """

    def __init__(self, account_id=None, ad_extension=None):
        if ad_extension and not isinstance(ad_extension, _VideoAdExtension):
            raise ValueError('The type of ad_extension is: {0}, should be: {1}'.format(
                type(ad_extension),
                'VideoAdExtension'
            ))
        super(BulkVideoAdExtension, self).__init__(
            account_id=account_id,
            ad_extension=ad_extension
        )

    @property
    def video_ad_extension(self):
        """ The video ad extension.

        see Video Ad Extension at https://go.microsoft.com/fwlink/?linkid=846127.
        """

        return self._ad_extension

    @video_ad_extension.setter
    def video_ad_extension(self, value):
        self._ad_extension = value

    _MAPPINGS = [
        _SimpleBulkMapping(
            header=_StringTable.Name,
            field_to_csv=lambda c: bulk_optional_str(c.video_ad_extension.Name, c.video_ad_extension.Id),
            csv_to_field=lambda c, v: setattr(c.video_ad_extension, 'Name', v if v else '')
        ),
        _SimpleBulkMapping(
            header=_StringTable.DisplayText,
            field_to_csv=lambda c: bulk_optional_str(c.video_ad_extension.DisplayText, c.video_ad_extension.Id),
            csv_to_field=lambda c, v: setattr(c.video_ad_extension, 'DisplayText', v if v else '')
        ),
        _SimpleBulkMapping(
            header=_StringTable.AltText,
            field_to_csv=lambda c: bulk_str(c.video_ad_extension.AlternativeText),
            csv_to_field=lambda c, v: setattr(c.video_ad_extension, 'AlternativeText', v)
        ),
        _SimpleBulkMapping(
            header=_StringTable.ActionText,
            field_to_csv=lambda c: bulk_optional_str(c.video_ad_extension.ActionText, c.video_ad_extension.Id),
            csv_to_field=lambda c, v: setattr(c.video_ad_extension, 'ActionText', v if v else '')
        ),
        _SimpleBulkMapping(
            header=_StringTable.ThumbnailUrl,
            field_to_csv=lambda c: bulk_str(c.video_ad_extension.ThumbnailUrl),
            csv_to_field=lambda c, v: setattr(c.video_ad_extension, 'ThumbnailUrl', v)
        ),
        _SimpleBulkMapping(
            header=_StringTable.ThumbnailId,
            field_to_csv=lambda c: bulk_str(c.video_ad_extension.ThumbnailId),
            csv_to_field=lambda c, v: setattr(c.video_ad_extension, 'ThumbnailId', int(v) if v else None)
        ),
        _SimpleBulkMapping(
            header=_StringTable.VideoId,
            field_to_csv=lambda c: bulk_str(c.video_ad_extension.VideoId),
            csv_to_field=lambda c, v: setattr(c.video_ad_extension, 'VideoId', int(v) if v else None)
        ),        
        _SimpleBulkMapping(
            header=_StringTable.FinalUrlSuffix,
            field_to_csv=lambda c: bulk_optional_str(c.video_ad_extension.FinalUrlSuffix, c.video_ad_extension.Id),
            csv_to_field=lambda c, v: setattr(c.video_ad_extension, 'FinalUrlSuffix', v)
        ),
        _SimpleBulkMapping(
            header=_StringTable.TrackingTemplate,
            field_to_csv=lambda c: bulk_str(c.video_ad_extension.TrackingUrlTemplate),
            csv_to_field=lambda c, v: setattr(c.video_ad_extension, 'TrackingUrlTemplate', v if v else None)
        ),
        _SimpleBulkMapping(
            header=_StringTable.CustomParameter,
            field_to_csv=lambda c: field_to_csv_UrlCustomParameters(c.video_ad_extension),
            csv_to_field=lambda c, v: csv_to_field_UrlCustomParameters(c.video_ad_extension, v)
        ),
        _SimpleBulkMapping(
            header=_StringTable.FinalUrl,
            field_to_csv=lambda c: field_to_csv_Urls(c.video_ad_extension.FinalUrls, c.video_ad_extension.Id),
            csv_to_field=lambda c, v: csv_to_field_Urls(c.video_ad_extension.FinalUrls, v)
        ),
        _SimpleBulkMapping(
            header=_StringTable.FinalMobileUrl,
            field_to_csv=lambda c: field_to_csv_Urls(c.video_ad_extension.FinalMobileUrls, c.video_ad_extension.Id),
            csv_to_field=lambda c, v: csv_to_field_Urls(c.video_ad_extension.FinalMobileUrls, v)
        )
    ]

    def process_mappings_from_row_values(self, row_values):
        self.video_ad_extension = _CAMPAIGN_OBJECT_FACTORY_V13.create('VideoAdExtension')
        self.video_ad_extension.Type = 'VideoAdExtension'
        super(BulkVideoAdExtension, self).process_mappings_from_row_values(row_values)
        row_values.convert_to_entity(self, BulkVideoAdExtension._MAPPINGS)

    def process_mappings_to_row_values(self, row_values, exclude_readonly_data):
        self._validate_property_not_null(self.video_ad_extension, 'video_ad_extension')
        super(BulkVideoAdExtension, self).process_mappings_to_row_values(row_values, exclude_readonly_data)
        self.convert_to_values(row_values, BulkVideoAdExtension._MAPPINGS)


class BulkAccountVideoAdExtension(_BulkAccountAdExtensionAssociation):
    """ Represents an account level video ad extension.

    This class exposes properties that can be read and written
    as fields of the Account Video Ad Extension record in a bulk file.

    For more information, see Account Video Ad Extension at https://go.microsoft.com/fwlink/?linkid=846127.

    *See also:*

    * :class:`.BulkServiceManager`
    * :class:`.BulkOperation`
    * :class:`.BulkFileReader`
    * :class:`.BulkFileWriter`
    """

    pass


class BulkCampaignVideoAdExtension(_BulkCampaignAdExtensionAssociation):
    """ Represents a campaign level video ad extension.

    This class exposes properties that can be read and written
    as fields of the Campaign Video Ad Extension record in a bulk file.

    For more information, see Campaign Video Ad Extension at https://go.microsoft.com/fwlink/?linkid=846127.

    *See also:*

    * :class:`.BulkServiceManager`
    * :class:`.BulkOperation`
    * :class:`.BulkFileReader`
    * :class:`.BulkFileWriter`
    """

    pass

class BulkAdGroupVideoAdExtension(_BulkAdGroupAdExtensionAssociation):
    """ Represents an ad group level video ad extension.

    This class exposes properties that can be read and written
    as fields of the Ad Group Video Ad Extension record in a bulk file.

    For more information, see Ad Group Video Ad Extension at https://go.microsoft.com/fwlink/?linkid=846127.

    *See also:*

    * :class:`.BulkServiceManager`
    * :class:`.BulkOperation`
    * :class:`.BulkFileReader`
    * :class:`.BulkFileWriter`
    """

    pass



# coding: utf-8

"""
    CustomerBillingService

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import datetime
from pydantic import BaseModel, ConfigDict, Field, StrictBool, StrictFloat, StrictInt, StrictStr
from typing import Any, ClassVar, Dict, List, Optional, Union, Set
from openapi_client.models.billing.insertion_order_pending_changes_status import InsertionOrderPendingChangesStatus
from typing_extensions import Self

class InsertionOrderPendingChanges(BaseModel):
    """
    InsertionOrderPendingChanges
    """ # noqa: E501
    comment: Optional[StrictStr] = Field(default=None, alias="Comment")
    end_date: Optional[datetime] = Field(default=None, alias="EndDate")
    requested_by_user_id: Optional[StrictInt] = Field(default=None, alias="RequestedByUserId")
    modified_date_time: Optional[datetime] = Field(default=None, alias="ModifiedDateTime")
    notification_threshold: Optional[StrictFloat] = Field(default=None, alias="NotificationThreshold")
    reference_id: Optional[StrictStr] = Field(default=None, alias="ReferenceId")
    spend_cap_amount: Optional[StrictFloat] = Field(default=None, alias="SpendCapAmount")
    start_date: Optional[datetime] = Field(default=None, alias="StartDate")
    name: Optional[StrictStr] = Field(default=None, alias="Name")
    purchase_order: Optional[StrictStr] = Field(default=None, alias="PurchaseOrder")
    change_status: Optional[InsertionOrderPendingChangesStatus] = Field(default=None, alias="ChangeStatus")
    __properties: ClassVar[List[str]] = ["Comment", "EndDate", "RequestedByUserId", "ModifiedDateTime", "NotificationThreshold", "ReferenceId", "SpendCapAmount", "StartDate", "Name", "PurchaseOrder", "ChangeStatus"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )
	

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # set to None if comment (nullable) is None
        # and model_fields_set contains the field
        if self.comment is None and "comment" in self.model_fields_set:
            _dict['Comment'] = None

        # set to None if end_date (nullable) is None
        # and model_fields_set contains the field
        if self.end_date is None and "end_date" in self.model_fields_set:
            _dict['EndDate'] = None

        # set to None if requested_by_user_id (nullable) is None
        # and model_fields_set contains the field
        if self.requested_by_user_id is None and "requested_by_user_id" in self.model_fields_set:
            _dict['RequestedByUserId'] = None

        # set to None if modified_date_time (nullable) is None
        # and model_fields_set contains the field
        if self.modified_date_time is None and "modified_date_time" in self.model_fields_set:
            _dict['ModifiedDateTime'] = None

        # set to None if notification_threshold (nullable) is None
        # and model_fields_set contains the field
        if self.notification_threshold is None and "notification_threshold" in self.model_fields_set:
            _dict['NotificationThreshold'] = None

        # set to None if reference_id (nullable) is None
        # and model_fields_set contains the field
        if self.reference_id is None and "reference_id" in self.model_fields_set:
            _dict['ReferenceId'] = None

        # set to None if spend_cap_amount (nullable) is None
        # and model_fields_set contains the field
        if self.spend_cap_amount is None and "spend_cap_amount" in self.model_fields_set:
            _dict['SpendCapAmount'] = None

        # set to None if start_date (nullable) is None
        # and model_fields_set contains the field
        if self.start_date is None and "start_date" in self.model_fields_set:
            _dict['StartDate'] = None

        # set to None if name (nullable) is None
        # and model_fields_set contains the field
        if self.name is None and "name" in self.model_fields_set:
            _dict['Name'] = None

        # set to None if purchase_order (nullable) is None
        # and model_fields_set contains the field
        if self.purchase_order is None and "purchase_order" in self.model_fields_set:
            _dict['PurchaseOrder'] = None

        # set to None if change_status (nullable) is None
        # and model_fields_set contains the field
        if self.change_status is None and "change_status" in self.model_fields_set:
            _dict['ChangeStatus'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of InsertionOrderPendingChanges from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "Comment": obj.get("Comment") if obj.get("Comment") is not None else None,
                        "EndDate": obj.get("EndDate") if obj.get("EndDate") is not None else None,
                        "RequestedByUserId": obj.get("RequestedByUserId") if obj.get("RequestedByUserId") is not None else None,
                        "ModifiedDateTime": obj.get("ModifiedDateTime") if obj.get("ModifiedDateTime") is not None else None,
                        "NotificationThreshold": obj.get("NotificationThreshold") if obj.get("NotificationThreshold") is not None else None,
                        "ReferenceId": obj.get("ReferenceId") if obj.get("ReferenceId") is not None else None,
                        "SpendCapAmount": obj.get("SpendCapAmount") if obj.get("SpendCapAmount") is not None else None,
                        "StartDate": obj.get("StartDate") if obj.get("StartDate") is not None else None,
                        "Name": obj.get("Name") if obj.get("Name") is not None else None,
                        "PurchaseOrder": obj.get("PurchaseOrder") if obj.get("PurchaseOrder") is not None else None,
                        "ChangeStatus": obj.get("ChangeStatus") if obj.get("ChangeStatus") is not None else None
        })
        return _obj



from abc import ABCMeta, abstractproperty


from bingads.internal.error_messages import _ErrorMessages
from bingads.v13.internal.bulk.bulk_object import _BulkObject


class BulkEntity(_BulkObject, metaclass=ABCMeta):
    """ The abstract base class for all bulk entities that can be read or written in a bulk file.

    For more information, see Bulk File Schema at https://go.microsoft.com/fwlink/?linkid=846127.

    *See also:*

    * :class:`.BulkServiceManager`
    * :class:`.BulkOperation`
    * :class:`.BulkFileReader`
    * :class:`.BulkFileWriter`
    """

    @abstractproperty
    def has_errors(self):
        """ Determines whether the bulk entity has associated errors.

        :rtype: bool
        """

        raise NotImplementedError()

    @abstractproperty
    def last_modified_time(self):
        """ Gets the last modified time for the entity.

        :rtype: :class:`datetime.datetime`
        """

        raise NotImplementedError()

    def _validate_property_not_null(self, property_value, property_name):
        if property_value is None:
            raise ValueError(_ErrorMessages.get_property_must_not_be_null_message(type(self).__name__, property_name))

    def _validate_list_not_null_or_empty(self, list_object, list_value, property_name):
        self._validate_property_not_null(list_object, property_name)
        if not list_value:
            raise ValueError(_ErrorMessages.get_list_must_not_be_null_or_empty(type(self).__name__, property_name))



# coding: utf-8

"""
    AdInsightService

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import datetime
from pydantic import BaseModel, ConfigDict, Field, StrictBool, StrictFloat, StrictInt, StrictStr
from typing import Any, ClassVar, Dict, List, Optional, Union, Set
from openapi_client.models.adinsight.currency import Currency
from openapi_client.models.adinsight.decimal_rounded_result import DecimalRoundedResult
from openapi_client.models.adinsight.range_result_of_decimal_rounded_result import RangeResultOfDecimalRoundedResult
from openapi_client.models.adinsight.range_result_ofdouble import RangeResultOfdouble
from typing_extensions import Self

class GetAudienceFullEstimationResponse(BaseModel):
    """
    GetAudienceFullEstimationResponse
    """ # noqa: E501
    est_impression: Optional[RangeResultOfDecimalRoundedResult] = Field(default=None, alias="EstImpression")
    est_audience_size: Optional[RangeResultOfDecimalRoundedResult] = Field(default=None, alias="EstAudienceSize")
    est_click: Optional[RangeResultOfDecimalRoundedResult] = Field(default=None, alias="EstClick")
    est_spend: Optional[RangeResultOfDecimalRoundedResult] = Field(default=None, alias="EstSpend")
    est_cost_per_event: Optional[RangeResultOfDecimalRoundedResult] = Field(default=None, alias="EstCostPerEvent")
    est_ctr: Optional[RangeResultOfdouble] = Field(default=None, alias="EstCTR")
    suggested_bid: Optional[StrictFloat] = Field(default=None, alias="SuggestedBid")
    suggested_budget: Optional[StrictFloat] = Field(default=None, alias="SuggestedBudget")
    events_lost_to_bid: Optional[StrictInt] = Field(default=None, alias="EventsLostToBid")
    events_lost_to_budget: Optional[StrictInt] = Field(default=None, alias="EventsLostToBudget")
    est_reach_audience_size: Optional[DecimalRoundedResult] = Field(default=None, alias="EstReachAudienceSize")
    est_reach_impression: Optional[DecimalRoundedResult] = Field(default=None, alias="EstReachImpression")
    currency: Optional[Currency] = Field(default=None, alias="Currency")
    __properties: ClassVar[List[str]] = ["EstImpression", "EstAudienceSize", "EstClick", "EstSpend", "EstCostPerEvent", "EstCTR", "SuggestedBid", "SuggestedBudget", "EventsLostToBid", "EventsLostToBudget", "EstReachAudienceSize", "EstReachImpression", "Currency"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )
	

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of est_impression
        if self.est_impression:
            _dict['EstImpression'] = self.est_impression.to_dict()
        # override the default output from pydantic by calling `to_dict()` of est_audience_size
        if self.est_audience_size:
            _dict['EstAudienceSize'] = self.est_audience_size.to_dict()
        # override the default output from pydantic by calling `to_dict()` of est_click
        if self.est_click:
            _dict['EstClick'] = self.est_click.to_dict()
        # override the default output from pydantic by calling `to_dict()` of est_spend
        if self.est_spend:
            _dict['EstSpend'] = self.est_spend.to_dict()
        # override the default output from pydantic by calling `to_dict()` of est_cost_per_event
        if self.est_cost_per_event:
            _dict['EstCostPerEvent'] = self.est_cost_per_event.to_dict()
        # override the default output from pydantic by calling `to_dict()` of est_ctr
        if self.est_ctr:
            _dict['EstCTR'] = self.est_ctr.to_dict()
        # override the default output from pydantic by calling `to_dict()` of est_reach_audience_size
        if self.est_reach_audience_size:
            _dict['EstReachAudienceSize'] = self.est_reach_audience_size.to_dict()
        # override the default output from pydantic by calling `to_dict()` of est_reach_impression
        if self.est_reach_impression:
            _dict['EstReachImpression'] = self.est_reach_impression.to_dict()
        # set to None if est_impression (nullable) is None
        # and model_fields_set contains the field
        if self.est_impression is None and "est_impression" in self.model_fields_set:
            _dict['EstImpression'] = None

        # set to None if est_audience_size (nullable) is None
        # and model_fields_set contains the field
        if self.est_audience_size is None and "est_audience_size" in self.model_fields_set:
            _dict['EstAudienceSize'] = None

        # set to None if est_click (nullable) is None
        # and model_fields_set contains the field
        if self.est_click is None and "est_click" in self.model_fields_set:
            _dict['EstClick'] = None

        # set to None if est_spend (nullable) is None
        # and model_fields_set contains the field
        if self.est_spend is None and "est_spend" in self.model_fields_set:
            _dict['EstSpend'] = None

        # set to None if est_cost_per_event (nullable) is None
        # and model_fields_set contains the field
        if self.est_cost_per_event is None and "est_cost_per_event" in self.model_fields_set:
            _dict['EstCostPerEvent'] = None

        # set to None if est_ctr (nullable) is None
        # and model_fields_set contains the field
        if self.est_ctr is None and "est_ctr" in self.model_fields_set:
            _dict['EstCTR'] = None

        # set to None if suggested_bid (nullable) is None
        # and model_fields_set contains the field
        if self.suggested_bid is None and "suggested_bid" in self.model_fields_set:
            _dict['SuggestedBid'] = None

        # set to None if suggested_budget (nullable) is None
        # and model_fields_set contains the field
        if self.suggested_budget is None and "suggested_budget" in self.model_fields_set:
            _dict['SuggestedBudget'] = None

        # set to None if events_lost_to_bid (nullable) is None
        # and model_fields_set contains the field
        if self.events_lost_to_bid is None and "events_lost_to_bid" in self.model_fields_set:
            _dict['EventsLostToBid'] = None

        # set to None if events_lost_to_budget (nullable) is None
        # and model_fields_set contains the field
        if self.events_lost_to_budget is None and "events_lost_to_budget" in self.model_fields_set:
            _dict['EventsLostToBudget'] = None

        # set to None if est_reach_audience_size (nullable) is None
        # and model_fields_set contains the field
        if self.est_reach_audience_size is None and "est_reach_audience_size" in self.model_fields_set:
            _dict['EstReachAudienceSize'] = None

        # set to None if est_reach_impression (nullable) is None
        # and model_fields_set contains the field
        if self.est_reach_impression is None and "est_reach_impression" in self.model_fields_set:
            _dict['EstReachImpression'] = None

        # set to None if currency (nullable) is None
        # and model_fields_set contains the field
        if self.currency is None and "currency" in self.model_fields_set:
            _dict['Currency'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of GetAudienceFullEstimationResponse from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "EstImpression": RangeResultOfDecimalRoundedResult.from_dict(obj["EstImpression"]) if obj.get("EstImpression") is not None else None,
                        "EstAudienceSize": RangeResultOfDecimalRoundedResult.from_dict(obj["EstAudienceSize"]) if obj.get("EstAudienceSize") is not None else None,
                        "EstClick": RangeResultOfDecimalRoundedResult.from_dict(obj["EstClick"]) if obj.get("EstClick") is not None else None,
                        "EstSpend": RangeResultOfDecimalRoundedResult.from_dict(obj["EstSpend"]) if obj.get("EstSpend") is not None else None,
                        "EstCostPerEvent": RangeResultOfDecimalRoundedResult.from_dict(obj["EstCostPerEvent"]) if obj.get("EstCostPerEvent") is not None else None,
                        "EstCTR": RangeResultOfdouble.from_dict(obj["EstCTR"]) if obj.get("EstCTR") is not None else None,
                        "SuggestedBid": obj.get("SuggestedBid") if obj.get("SuggestedBid") is not None else None,
                        "SuggestedBudget": obj.get("SuggestedBudget") if obj.get("SuggestedBudget") is not None else None,
                        "EventsLostToBid": obj.get("EventsLostToBid") if obj.get("EventsLostToBid") is not None else None,
                        "EventsLostToBudget": obj.get("EventsLostToBudget") if obj.get("EventsLostToBudget") is not None else None,
                        "EstReachAudienceSize": DecimalRoundedResult.from_dict(obj["EstReachAudienceSize"]) if obj.get("EstReachAudienceSize") is not None else None,
                        "EstReachImpression": DecimalRoundedResult.from_dict(obj["EstReachImpression"]) if obj.get("EstReachImpression") is not None else None,
                        "Currency": obj.get("Currency") if obj.get("Currency") is not None else None
        })
        return _obj



from abc import ABCMeta, abstractmethod

from bingads.v13.internal.bulk.string_table import _StringTable
from bingads.v13.internal.bulk.mappings import _SimpleBulkMapping
from bingads.v13.bulk.entities.bulk_entity import BulkEntity
from bingads.v13.bulk.entities.bulk_error import BulkError
from bingads.v13.internal.extensions import *


class _SingleRecordBulkEntity(BulkEntity,metaclass=ABCMeta):
    def __init__(self):
        self._client_id = None
        self._last_modified_time = None
        self._errors = None

    @property
    def client_id(self):
        return self._client_id

    @client_id.setter
    def client_id(self, client_id):
        self._client_id = client_id

    @property
    def last_modified_time(self):
        return self._last_modified_time

    @property
    def errors(self):
        """ A list of :class:`BulkError` details in a separate bulk record that corresponds to the record of a :class:`.BulkEntity` derived instance.

        :rtype: list[BulkError] or None
        """
        return self._errors

    @property
    def has_errors(self):
        return True if self.errors is not None and self.errors else False

    _MAPPINGS = [
        _SimpleBulkMapping(
            header=_StringTable.ClientId,
            field_to_csv=lambda c: c.client_id,
            csv_to_field=lambda c, v: setattr(c, '_client_id', v),
        ),
        _SimpleBulkMapping(
            header=_StringTable.LastModifiedTime,
            field_to_csv=lambda c: bulk_datetime_str(c.last_modified_time),
            csv_to_field=lambda c, v: setattr(
                c,
                '_last_modified_time',
                parse_datetime(v) if v else None),
        ),
    ]

    def read_from_row_values(self, row_values):
        row_values.convert_to_entity(self, _SingleRecordBulkEntity._MAPPINGS)
        self.process_mappings_from_row_values(row_values)

    def write_to_row_values(self, row_values, exclude_readonly_data):
        self.convert_to_values(row_values, _SingleRecordBulkEntity._MAPPINGS)
        self.process_mappings_to_row_values(row_values, exclude_readonly_data)

    def read_related_data_from_stream(self, stream_reader):
        self.read_additional_data(stream_reader)
        self._read_errors(stream_reader)

    def write_to_stream(self, row_writer, exclude_readonly_data):
        row_writer.write_object_row(self, exclude_readonly_data)
        if not exclude_readonly_data:
            self._write_errors(row_writer)
            self.write_additional_data(row_writer)

    def _write_errors(self, row_writer):
        if self.has_errors:
            for error in self.errors:
                row_writer.write_object_row(error)

    def write_additional_data(self, row_writer):
        pass

    @abstractmethod
    def process_mappings_to_row_values(self, row_values, exclude_readonly_data):
        """ Process specific entity mappings to CSV values. Must be implemented by each entity.

        :param row_values: Row values.
        :type row_values: _RowValues
        :param exclude_readonly_data: excludeReadonlyData indicates whether readonly data should be written (such as errors, performance data etc.)
        :type exclude_readonly_data: bool
        """

        raise NotImplementedError()

    @abstractmethod
    def process_mappings_from_row_values(self, row_values):
        """ Process specific entity mappings from CSV values. Must be implemented by each entity.

        :param row_values:
        :type row_values: _RowValues
        """

        raise NotImplementedError()

    @abstractmethod
    def read_additional_data(self, stream_reader):
        pass

    def _read_errors(self, stream_reader):
        errors = []
        success, error = stream_reader.try_read(BulkError)
        while success:
            error.entity = self
            errors.append(error)
            success, error = stream_reader.try_read(BulkError)
        self._errors = errors

    def enclose_in_multiline_entity(self):
        return super(_SingleRecordBulkEntity, self).enclose_in_multiline_entity()

    @property
    def can_enclose_in_multiline_entity(self):
        return super(_SingleRecordBulkEntity, self).can_enclose_in_multiline_entity



# coding: utf-8

"""
    CampaignManagementService

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import datetime
from pydantic import BaseModel, ConfigDict, Field, StrictBool, StrictFloat, StrictInt, StrictStr
from typing import Any, ClassVar, Dict, List, Optional, Union, Set
from openapi_client.models.campaign.import_entity_statistics import ImportEntityStatistics
from openapi_client.models.campaign.import_job import ImportJob
from openapi_client.models.campaign.key_value_pair_ofstring_andstring import KeyValuePairOfstringAndstring
from typing_extensions import Self

class ImportResult(BaseModel):
    """
    ImportResult
    """ # noqa: E501
    id: Optional[StrictStr] = Field(default=None, alias="Id")
    start_time_in_utc: Optional[datetime] = Field(default=None, alias="StartTimeInUTC")
    status: Optional[StrictStr] = Field(default=None, alias="Status")
    import_job: Optional[ImportJob] = Field(default=None, alias="ImportJob")
    entity_statistics: Optional[List[Optional[ImportEntityStatistics]]] = Field(default=None, alias="EntityStatistics")
    error_log_url: Optional[StrictStr] = Field(default=None, alias="ErrorLogUrl")
    forward_compatibility_map: Optional[List[Optional[KeyValuePairOfstringAndstring]]] = Field(default=None, alias="ForwardCompatibilityMap")
    __properties: ClassVar[List[str]] = ["Id", "StartTimeInUTC", "Status", "ImportJob", "EntityStatistics", "ErrorLogUrl", "ForwardCompatibilityMap"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )
	

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of import_job
        if self.import_job:
            _dict['ImportJob'] = self.import_job.to_dict()
        # override the default output from pydantic by calling `to_dict()` of each item in entity_statistics (list)
        _items = []
        if self.entity_statistics:
            for _item_entity_statistics in self.entity_statistics:
                if _item_entity_statistics:
                    _items.append(_item_entity_statistics.to_dict())
            _dict['EntityStatistics'] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in forward_compatibility_map (list)
        _items = []
        if self.forward_compatibility_map:
            for _item_forward_compatibility_map in self.forward_compatibility_map:
                if _item_forward_compatibility_map:
                    _items.append(_item_forward_compatibility_map.to_dict())
            _dict['ForwardCompatibilityMap'] = _items
        # set to None if id (nullable) is None
        # and model_fields_set contains the field
        if self.id is None and "id" in self.model_fields_set:
            _dict['Id'] = None

        # set to None if start_time_in_utc (nullable) is None
        # and model_fields_set contains the field
        if self.start_time_in_utc is None and "start_time_in_utc" in self.model_fields_set:
            _dict['StartTimeInUTC'] = None

        # set to None if status (nullable) is None
        # and model_fields_set contains the field
        if self.status is None and "status" in self.model_fields_set:
            _dict['Status'] = None

        # set to None if import_job (nullable) is None
        # and model_fields_set contains the field
        if self.import_job is None and "import_job" in self.model_fields_set:
            _dict['ImportJob'] = None

        # set to None if entity_statistics (nullable) is None
        # and model_fields_set contains the field
        if self.entity_statistics is None and "entity_statistics" in self.model_fields_set:
            _dict['EntityStatistics'] = None

        # set to None if error_log_url (nullable) is None
        # and model_fields_set contains the field
        if self.error_log_url is None and "error_log_url" in self.model_fields_set:
            _dict['ErrorLogUrl'] = None

        # set to None if forward_compatibility_map (nullable) is None
        # and model_fields_set contains the field
        if self.forward_compatibility_map is None and "forward_compatibility_map" in self.model_fields_set:
            _dict['ForwardCompatibilityMap'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of ImportResult from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "Id": obj.get("Id") if obj.get("Id") is not None else None,
                        "StartTimeInUTC": obj.get("StartTimeInUTC") if obj.get("StartTimeInUTC") is not None else None,
                        "Status": obj.get("Status") if obj.get("Status") is not None else None,
                        "ImportJob": ImportJob.from_dict(obj["ImportJob"]) if obj.get("ImportJob") is not None else None,
                        "EntityStatistics": [ImportEntityStatistics.from_dict(_item) for _item in obj["EntityStatistics"]] if obj.get("EntityStatistics") is not None else None,
                        "ErrorLogUrl": obj.get("ErrorLogUrl") if obj.get("ErrorLogUrl") is not None else None,
                        "ForwardCompatibilityMap": [KeyValuePairOfstringAndstring.from_dict(_item) for _item in obj["ForwardCompatibilityMap"]] if obj.get("ForwardCompatibilityMap") is not None else None
        })
        return _obj



from bingads.v13.internal.bulk.mappings import _SimpleBulkMapping
from bingads.v13.internal.bulk.string_table import _StringTable
from bingads.service_client import _CAMPAIGN_OBJECT_FACTORY_V13

from bingads.v13.internal.extensions import *
from .common import _BulkAdExtensionBase
from .common import _BulkCampaignAdExtensionAssociation
from .common import _BulkAdGroupAdExtensionAssociation
from .common import _BulkAccountAdExtensionAssociation

_FilterLinkAdExtension = type(_CAMPAIGN_OBJECT_FACTORY_V13.create('FilterLinkAdExtension'))


class BulkFilterLinkAdExtension(_BulkAdExtensionBase):
    """ Represents a filter link ad extension.

    This class exposes the :attr:`filter_link_ad_extension` property that can be read and written
    as fields of the Filter Link Ad Extension record in a bulk file.

    For more information, see Filter Link Ad Extension at https://go.microsoft.com/fwlink/?linkid=846127.

    *See also:*

    * :class:`.BulkServiceManager`
    * :class:`.BulkOperation`
    * :class:`.BulkFileReader`
    * :class:`.BulkFileWriter`
    """

    def __init__(self, account_id=None, ad_extension=None):
        if ad_extension and not isinstance(ad_extension, _FilterLinkAdExtension):
            raise ValueError('The type of ad_extension is: {0}, should be: {1}'.format(
                type(ad_extension),
                'FilterLinkAdExtension'
            ))
        super(BulkFilterLinkAdExtension, self).__init__(
            account_id=account_id,
            ad_extension=ad_extension
        )

    @property
    def filter_link_ad_extension(self):
        """ The filter link ad extension.

        see Filter Link Ad Extension at https://go.microsoft.com/fwlink/?linkid=846127.
        """

        return self._ad_extension

    @filter_link_ad_extension.setter
    def filter_link_ad_extension(self, value):
        self._ad_extension = value

    _MAPPINGS = [
        _SimpleBulkMapping(
            header=_StringTable.AdExtensionHeaderType,
            field_to_csv=lambda c: bulk_str(c.filter_link_ad_extension.AdExtensionHeaderType),
            csv_to_field=lambda c, v: setattr(c.filter_link_ad_extension, 'AdExtensionHeaderType', v)
        ),
        _SimpleBulkMapping(
            header=_StringTable.Language,
            field_to_csv=lambda c: c.filter_link_ad_extension.Language,
            csv_to_field=lambda c, v: setattr(c.filter_link_ad_extension, 'Language', v)
        ),
        _SimpleBulkMapping(
            header=_StringTable.Texts,
            field_to_csv=lambda c: field_to_csv_delimited_strings(c.filter_link_ad_extension.Texts),
            csv_to_field=lambda c, v: csv_to_field_delimited_strings(c.filter_link_ad_extension.Texts, v)
        ),
        _SimpleBulkMapping(
            header=_StringTable.TrackingTemplate,
            field_to_csv=lambda c: bulk_str(c.filter_link_ad_extension.TrackingUrlTemplate),
            csv_to_field=lambda c, v: setattr(c.filter_link_ad_extension, 'TrackingUrlTemplate', v if v else None)
        ),
        _SimpleBulkMapping(
            header=_StringTable.CustomParameter,
            field_to_csv=lambda c: field_to_csv_UrlCustomParameters(c.filter_link_ad_extension),
            csv_to_field=lambda c, v: csv_to_field_UrlCustomParameters(c.filter_link_ad_extension, v)
        ),
        _SimpleBulkMapping(
            header=_StringTable.FinalUrl,
            field_to_csv=lambda c: field_to_csv_Urls(c.filter_link_ad_extension.FinalUrls, c.filter_link_ad_extension.Id),
            csv_to_field=lambda c, v: csv_to_field_Urls(c.filter_link_ad_extension.FinalUrls, v)
        ),
        _SimpleBulkMapping(
            header=_StringTable.FinalMobileUrl,
            field_to_csv=lambda c: field_to_csv_Urls(c.filter_link_ad_extension.FinalMobileUrls, c.filter_link_ad_extension.Id),
            csv_to_field=lambda c, v: csv_to_field_Urls(c.filter_link_ad_extension.FinalMobileUrls, v)
        ),
        _SimpleBulkMapping(
            header=_StringTable.FinalUrlSuffix,
            field_to_csv=lambda c: bulk_optional_str(c.filter_link_ad_extension.FinalUrlSuffix, c.filter_link_ad_extension.Id),
            csv_to_field=lambda c, v: setattr(c.filter_link_ad_extension, 'FinalUrlSuffix', v)
        )
    ]

    def process_mappings_from_row_values(self, row_values):
        self.filter_link_ad_extension = _CAMPAIGN_OBJECT_FACTORY_V13.create('FilterLinkAdExtension')
        self.filter_link_ad_extension.Type = 'FilterLinkAdExtension'
        super(BulkFilterLinkAdExtension, self).process_mappings_from_row_values(row_values)
        row_values.convert_to_entity(self, BulkFilterLinkAdExtension._MAPPINGS)

    def process_mappings_to_row_values(self, row_values, exclude_readonly_data):
        self._validate_property_not_null(self.filter_link_ad_extension, 'filter_link_ad_extension')
        super(BulkFilterLinkAdExtension, self).process_mappings_to_row_values(row_values, exclude_readonly_data)
        self.convert_to_values(row_values, BulkFilterLinkAdExtension._MAPPINGS)


class BulkAccountFilterLinkAdExtension(_BulkAccountAdExtensionAssociation):
    """ Represents an account level filter link ad extension.

    This class exposes properties that can be read and written
    as fields of the Account Filter Link Ad Extension record in a bulk file.

    For more information, see Account Filter Link Ad Extension at https://go.microsoft.com/fwlink/?linkid=846127.

    *See also:*

    * :class:`.BulkServiceManager`
    * :class:`.BulkOperation`
    * :class:`.BulkFileReader`
    * :class:`.BulkFileWriter`
    """

    pass


class BulkCampaignFilterLinkAdExtension(_BulkCampaignAdExtensionAssociation):
    """ Represents a campaign level filter link ad extension.

    This class exposes properties that can be read and written
    as fields of the Campaign Filter Link Ad Extension record in a bulk file.

    For more information, see Campaign Filter Link Ad Extension at https://go.microsoft.com/fwlink/?linkid=846127.

    *See also:*

    * :class:`.BulkServiceManager`
    * :class:`.BulkOperation`
    * :class:`.BulkFileReader`
    * :class:`.BulkFileWriter`
    """

    pass

class BulkAdGroupFilterLinkAdExtension(_BulkAdGroupAdExtensionAssociation):
    """ Represents an ad group level filter link ad extension.

    This class exposes properties that can be read and written
    as fields of the Ad Group Filter Link Ad Extension record in a bulk file.

    For more information, see Ad Group Filter Link Ad Extension at https://go.microsoft.com/fwlink/?linkid=846127.

    *See also:*

    * :class:`.BulkServiceManager`
    * :class:`.BulkOperation`
    * :class:`.BulkFileReader`
    * :class:`.BulkFileWriter`
    """

    pass



__author__ = 'Bing Ads SDK Team'
__email__ = 'bing_ads_sdk@microsoft.com'

from .common import *

from .bulk_error import *
from .bulk_entity import *
from .bid_suggestion_data import *
from .unknown_bulk_entity import *
from .bulk_account import *
from .bulk_budget import *
from .bulk_campaign import *
from .bulk_ad_group import *
from .bulk_keyword import *
from .bulk_campaign_product_scope import *
from .bulk_ad_group_product_partition import *
from .bulk_campaign_negative_store_criterion import *
from .bulk_campaign_negative_dynamic_search_ad_target import *
from .bulk_ad_group_dynamic_search_ad_target import *
from .bulk_ad_group_negative_dynamic_search_ad_target import *
from .bulk_bid_strategy import *
from .bulk_ad_group_criterion import *
from .bulk_ad_group_hotel_listing_group import *

from .ad_extensions import *
from .bulk_ads import *
from .bulk_negative_keywords import *
from .bulk_negative_sites import *
from .audiences import *
from .target_criterions import *
from .labels import *
from .bulk_offline_conversion import *
from .bulk_online_conversion_adjustment import *
from .bulk_experiment import *
from .bulk_image import *
from .bulk_video import *
from .feeds import *
from .bulk_campaign_conversion_goal import *
from .bulk_ad_customizer_attribute import *
from .bulk_ad_customizer_attribute_entity_base import *
from .bulk_ad_customizer_attribute_campaign import *
from .bulk_ad_customizer_attribute_ad_group import *
from .bulk_ad_customizer_attribute_keyword import *

from .bulk_asset_group import *
from .bulk_audience_group import *
from .bulk_asset_group_listing_group import *
from .bulk_asset_group_url_target import *
from .bulk_audience_group_asset_group_association import *
from .bulk_campaign_negative_webpage import *
from .bulk_seasonality_adjustment import *
from .bulk_data_exclusion import *
from .bulk_account_negative_keyword_list import *
from .bulk_asset_group_search_theme import *
from .bulk_campaign_brand_list_association import *
from .bulk_brand_item import *
from .bulk_brand_list import *
from .bulk_new_customer_acquisition_goal import *
from .goals import *



# coding: utf-8

"""
    CustomerManagementService

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import datetime
from pydantic import BaseModel, ConfigDict, Field, StrictBool, StrictFloat, StrictInt, StrictStr
from typing import Any, ClassVar, Dict, List, Optional, Union, Set
from typing_extensions import Self

class UpdateUserRolesRequest(BaseModel):
    """
    UpdateUserRolesRequest
    """ # noqa: E501
    customer_id: Optional[StrictStr] = Field(default=None, alias="CustomerId")
    user_id: Optional[StrictStr] = Field(default=None, alias="UserId")
    new_role_id: Optional[StrictInt] = Field(default=None, alias="NewRoleId")
    new_account_ids: Optional[List[StrictStr]] = Field(default=None, alias="NewAccountIds")
    new_customer_ids: Optional[List[StrictStr]] = Field(default=None, alias="NewCustomerIds")
    delete_role_id: Optional[StrictInt] = Field(default=None, alias="DeleteRoleId")
    delete_account_ids: Optional[List[StrictStr]] = Field(default=None, alias="DeleteAccountIds")
    delete_customer_ids: Optional[List[StrictStr]] = Field(default=None, alias="DeleteCustomerIds")
    __properties: ClassVar[List[str]] = ["CustomerId", "UserId", "NewRoleId", "NewAccountIds", "NewCustomerIds", "DeleteRoleId", "DeleteAccountIds", "DeleteCustomerIds"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )
	

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # set to None if customer_id (nullable) is None
        # and model_fields_set contains the field
        if self.customer_id is None and "customer_id" in self.model_fields_set:
            _dict['CustomerId'] = None

        # set to None if user_id (nullable) is None
        # and model_fields_set contains the field
        if self.user_id is None and "user_id" in self.model_fields_set:
            _dict['UserId'] = None

        # set to None if new_role_id (nullable) is None
        # and model_fields_set contains the field
        if self.new_role_id is None and "new_role_id" in self.model_fields_set:
            _dict['NewRoleId'] = None

        # set to None if new_account_ids (nullable) is None
        # and model_fields_set contains the field
        if self.new_account_ids is None and "new_account_ids" in self.model_fields_set:
            _dict['NewAccountIds'] = None

        # set to None if new_customer_ids (nullable) is None
        # and model_fields_set contains the field
        if self.new_customer_ids is None and "new_customer_ids" in self.model_fields_set:
            _dict['NewCustomerIds'] = None

        # set to None if delete_role_id (nullable) is None
        # and model_fields_set contains the field
        if self.delete_role_id is None and "delete_role_id" in self.model_fields_set:
            _dict['DeleteRoleId'] = None

        # set to None if delete_account_ids (nullable) is None
        # and model_fields_set contains the field
        if self.delete_account_ids is None and "delete_account_ids" in self.model_fields_set:
            _dict['DeleteAccountIds'] = None

        # set to None if delete_customer_ids (nullable) is None
        # and model_fields_set contains the field
        if self.delete_customer_ids is None and "delete_customer_ids" in self.model_fields_set:
            _dict['DeleteCustomerIds'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of UpdateUserRolesRequest from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "CustomerId": obj.get("CustomerId") if obj.get("CustomerId") is not None else None,
                        "UserId": obj.get("UserId") if obj.get("UserId") is not None else None,
                        "NewRoleId": obj.get("NewRoleId") if obj.get("NewRoleId") is not None else None,
                        "NewAccountIds": obj.get("NewAccountIds"),
                        "NewCustomerIds": obj.get("NewCustomerIds"),
                        "DeleteRoleId": obj.get("DeleteRoleId") if obj.get("DeleteRoleId") is not None else None,
                        "DeleteAccountIds": obj.get("DeleteAccountIds"),
                        "DeleteCustomerIds": obj.get("DeleteCustomerIds")
        })
        return _obj



# coding: utf-8

"""
    CustomerBillingService

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import datetime
from pydantic import BaseModel, ConfigDict, Field, StrictBool, StrictFloat, StrictInt, StrictStr
from typing import Any, ClassVar, Dict, List, Optional, Union, Set
from openapi_client.models.billing.insertion_order_additional_field import InsertionOrderAdditionalField
from openapi_client.models.billing.order_by import OrderBy
from openapi_client.models.billing.paging import Paging
from openapi_client.models.billing.predicate import Predicate
from typing_extensions import Self

class SearchInsertionOrdersRequest(BaseModel):
    """
    SearchInsertionOrdersRequest
    """ # noqa: E501
    predicates: Optional[List[Optional[Predicate]]] = Field(default=None, alias="Predicates")
    ordering: Optional[List[Optional[OrderBy]]] = Field(default=None, alias="Ordering")
    page_info: Optional[Paging] = Field(default=None, alias="PageInfo")
    return_additional_fields: Optional[InsertionOrderAdditionalField] = Field(default=None, alias="ReturnAdditionalFields")
    __properties: ClassVar[List[str]] = ["Predicates", "Ordering", "PageInfo", "ReturnAdditionalFields"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )
	

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of each item in predicates (list)
        _items = []
        if self.predicates:
            for _item_predicates in self.predicates:
                if _item_predicates:
                    _items.append(_item_predicates.to_dict())
            _dict['Predicates'] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in ordering (list)
        _items = []
        if self.ordering:
            for _item_ordering in self.ordering:
                if _item_ordering:
                    _items.append(_item_ordering.to_dict())
            _dict['Ordering'] = _items
        # override the default output from pydantic by calling `to_dict()` of page_info
        if self.page_info:
            _dict['PageInfo'] = self.page_info.to_dict()
        # set to None if predicates (nullable) is None
        # and model_fields_set contains the field
        if self.predicates is None and "predicates" in self.model_fields_set:
            _dict['Predicates'] = None

        # set to None if ordering (nullable) is None
        # and model_fields_set contains the field
        if self.ordering is None and "ordering" in self.model_fields_set:
            _dict['Ordering'] = None

        # set to None if page_info (nullable) is None
        # and model_fields_set contains the field
        if self.page_info is None and "page_info" in self.model_fields_set:
            _dict['PageInfo'] = None

        # set to None if return_additional_fields (nullable) is None
        # and model_fields_set contains the field
        if self.return_additional_fields is None and "return_additional_fields" in self.model_fields_set:
            _dict['ReturnAdditionalFields'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of SearchInsertionOrdersRequest from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "Predicates": [Predicate.from_dict(_item) for _item in obj["Predicates"]] if obj.get("Predicates") is not None else None,
                        "Ordering": [OrderBy.from_dict(_item) for _item in obj["Ordering"]] if obj.get("Ordering") is not None else None,
                        "PageInfo": Paging.from_dict(obj["PageInfo"]) if obj.get("PageInfo") is not None else None,
                        "ReturnAdditionalFields": obj.get("ReturnAdditionalFields") if obj.get("ReturnAdditionalFields") is not None else None
        })
        return _obj
